/*
   installTheme; install and configure a blackbox theme.
   Copyright (C) 1999-2007 Will Eighteen: will@opaline.demon.co.uk

   This program assumes that the theme style and image files are in the
   ~/.blackhome/styles directory.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or   
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    See .blackhome/COPYING for full details.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>

extern char **environ;

/* limit for string copying - change for longer paths */
#define MAXCHARS 256

int filecopy(char *in, char *out);
int uninstall(char *removeTheme, char *topDir, char *removeImage);
int removeLinks(char *homeDirName);
int sysCall(const char *command);

int main(int argc, char * argv[])
{
   unsigned int count, ptr;
   int lineCount;
   unsigned int findLineNum;
   unsigned int themeCount; /* count chars in theme name */
   char textString[16*MAXCHARS];
   char line[2*MAXCHARS];	/* 2 more lines after 'rootCommand' in style file allowed */
   char styleRC[16*MAXCHARS];	/* existing style root command' in style file (if any) */
   char themeName[MAXCHARS];	    /* is there a longer one? */
   char imageName[MAXCHARS];
   char homeDirName[MAXCHARS];
   char workingFile[3*MAXCHARS];
   char embeddedTheme[MAXCHARS];
   char tempName[2*MAXCHARS];
   char shellFlags[2];
   char c;
   char inFile[3*MAXCHARS];
   char outFile[4*MAXCHARS];
   char appDefsName[MAXCHARS];   /* X application resource */
   char userDefsName[MAXCHARS];  /* theme-named copy of X app resource */
   int isSep;
   int isColour;
   int isExitLine;
   FILE *fileHandle, *fileHandle2;

   if (argc == 1) /* check for just 'installTheme' */
   {
      printf("blackhome-0.3.4.2: Will 18, 2007\n");
      printf("theme, themeName or imageName not found.\n");
      printf("cd to ~/.blackhome/config and use\n");
      printf("./install theme themeName imageName\n");
      printf("\nEnsure the theme style and image files are in the\n");
      printf("~/.blackhome/styles directory\n");
      return 1;
   }

   /* 'install cleanup ~' */
   if (argc == 3) /* uninstall home dir links */
   {  
      if (strlen(argv[1]) < MAXCHARS-1 )
      	 sprintf(themeName, "%s", argv[1]);
      else
      {
      	 printf("theme name %s is more than %u chars, rejecting...\n",
               	argv[1], MAXCHARS-1);
         return 1;
      }

      if (strlen(argv[2]) < MAXCHARS-1 )
      	 sprintf(homeDirName, "%s", argv[2]);
      else
      {
      	printf("home dir %s is more than %u chars, rejecting...\n",
               	argv[2], MAXCHARS-1);
         return 1;
      }

      if (strcmp(themeName, "cleanup") == 0)
      { 
         removeLinks(homeDirName);
         return 0;
      }
   }

   sprintf(textString, "%s", argv[1]);

   if ((strcmp(textString, "theme") == 0) || (strcmp(textString, "image") == 0) || (strcmp(textString, "xpm") == 0))
   {
      if (strlen(argv[2]) < MAXCHARS-1 )
         sprintf(themeName, "%s", argv[2]);
      else
      {
         printf("theme name %s more than %u chars, rejecting...\n", argv[2], MAXCHARS-1);
         return 1;
      }
   
      if (strlen(argv[3]) < MAXCHARS-1 )
         sprintf(imageName, "%s", argv[3]);
      else
      {
         printf("image name %s more than %u chars, rejecting...\n", argv[3], MAXCHARS-1);
         return 1;
      }

      if (strlen(argv[4]) < MAXCHARS-1 )
         sprintf(homeDirName, "%s", argv[4]);
      else
      {
         printf("path %s more than %u chars, rejecting...\n", argv[4], MAXCHARS-1);
         return 1;
      }
      
      printf("Theme name: %s\n", argv[2]); 
      printf("image name: %s\n", argv[3]); 
      printf("home directory: %s\n", argv[4]); 
   
      /* check args, 5th one is shell-supplied y/n */
      /* governing whether we want .Xresources/.Xdefaults built for theme */
      if (argc == 6) /* extra parameter is shell-supplied flag 'string' */
      {
         if(strlen(argv[5]) > 2)
         {
         	printf("error: only 2 flags allowed (each y or n) not %s\n", argv[5]);
            return 1;
         }
         else
            sprintf(shellFlags, "%s", argv[5]);
      }
   }
   else
   {
      if (strlen(argv[1]) < MAXCHARS-1 )
      	 sprintf(themeName, "%s", argv[1]);
      else
      {
      	 printf("theme name %s is more than %u chars, rejecting...\n",
               	argv[1], MAXCHARS-1);
         return 1;
      }

      if (strlen(argv[2]) < MAXCHARS-1 )
      	 sprintf(homeDirName, "%s", argv[2]);
      else
      {
      	printf("home dir %s is more than %u chars, rejecting...\n",
               	argv[2], MAXCHARS-1);
         return 1;
      }

      if (strlen(argv[3]) < MAXCHARS-1 )
      	 sprintf(imageName, "%s", argv[3]);
      else
      {
      	printf("image name %s is more than %u chars, rejecting...\n",
               	argv[3], MAXCHARS-1);
         return 1;
      }

      if (uninstall(themeName, homeDirName, imageName) == 1)
   	  {
         printf("failed uninstalling theme %s\n", themeName);
   	     return 1;
      }
   	  else
      {
   	     printf("uninstalled theme %s\n", themeName);
       	 return 0;
      }
   }

/* 1) Check for presence of stylesmenu, then its contents. */
/*    Refuse to set up two themes with the same name (confuses uninstall). */

   styleRC[0] = 0;
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/styles/stylesmenu");

   /* check for themeName in stylesmenu - if present, exit install */
   if ((fileHandle = fopen(workingFile, "r")) != NULL)
   {  /* some user styles are already installed */
      c = 0;
      lineCount = 0;

      do
      {
         fread(&c, 1, 1, fileHandle);        
         textString[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if (textString[0] == '[') /* theme line start */
            {
               count = 0;

               while (textString[count] != '(')
               count++; /* found start of theme name entry */

               count++;
               themeCount = 0;
               
               while (textString[count] != ')') 
               {
                  embeddedTheme[themeCount] = textString[count];
                  themeCount++;
                  count++;

               	if (count >= MAXCHARS)
                  {
                  	printf("theme name too long, rejecting\n");
                     return 1;
                  }
               }

               strcpy(tempName,themeName);
               embeddedTheme[themeCount] = 0x00;

               if (strcmp(embeddedTheme, tempName) == 0)
               {
                  printf("Theme %s is already installed!\n", themeName);
                  printf("Remove this first or rename the new theme\n");
                  printf("style and image files.\n");
                  return 1;
               }
            }

            strcpy(textString, "");
            lineCount = -1;
         }  /* end got current line */

         lineCount++;
      }  while (!feof(fileHandle));
   }

   fclose(fileHandle);

   printf("Setting up theme %s\n", themeName);
   strcpy(workingFile, homeDirName);
   strcat(workingFile,"/.blackhome/styles/");
   strcat(workingFile, themeName);

/* 2) check style file for presence of 'rootCommand' and transfer to control
      script <themeName>.sh */

   fileHandle = fopen(workingFile, "r+");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);
      return 1;
   }

	while ( fgets ( textString, sizeof ( textString ), fileHandle ) != NULL )
	{
		if ( strstr ( textString, "rootCommand" ) != NULL )
		{
			if (!((textString[0] == '!') || (textString[0] == '#')))
         {
				isSep = 0;
				isColour = 0;

				if (strstr(textString, "/") != NULL)
					isSep = 1;

				if (strstr(textString, "rgb:") != NULL)
					isColour = 1;

				if (strstr(textString, "rgbi:") != NULL)
					isColour = 1;

				if (strstr(textString, "-mod") != NULL)
					isColour = 1;

				/* remove odd dirs. If sep and not colour, is just a dir not colour spec */
				if (!((isSep == 1) && (isColour == 0)))
				{
					strcpy(styleRC, textString);
					strcpy(textString, "#");
					strcat(textString, styleRC );

					/* force run to end of file to pick up additional command
					lines*/
					while ( fgets ( line, sizeof ( line ), fileHandle ) != NULL )
						strcat (textString, line);

					/* rewind file and comment out existing line */
					fseek (fileHandle, -(strlen(textString)-1), SEEK_CUR);
					fputs(textString, fileHandle);

					/* if textString is 'rootCommand', we need to cut this to get
					   actual command for control.sh script */
					count = 0;

					while ((styleRC[count] == ' ') || (styleRC[count] == '	'))	/* find leading spaces */
						count ++;

					if (strstr(styleRC, "rootCommand:") != NULL )
						count += 12;

					while ((styleRC[count] == ' ') || (styleRC[count] == '	'))	/* now find trailing spaces */
						count ++;

					/* now get adjusted styleRC string*/
					ptr = 0;

					while ( count < strlen(styleRC))
					{
						textString[ptr] = styleRC[count];
						count++;
						ptr++;
					}

					textString[ptr] = 0;
					strcpy(styleRC, textString);
				}
			}
		}
	}

	fclose (fileHandle);

	if ((strstr (styleRC, "/usr") != NULL)
		|| (strstr(styleRC, "/share") != NULL)
		|| (strstr (styleRC, "blackbox") != NULL)
		|| (strstr(styleRC,"Blackbox") != NULL)
		|| (strstr(styleRC, "fluxbox") != NULL)
		|| (strstr(styleRC, "Fluxbox") != NULL))
		/* presume image in wm dir (if any of these are in the image name, fails) */
	   styleRC[0] = 0;	/* image file in wm dir, just use image name/w default
						   viewer */

/* 3) open style file ./blackhome/styles/themeName for append at end */
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);
      return 1;
   }
  
   /* add rootCommand to end of the style file: executes bash script */
   /* in ~/.blackhome/control */

   strcpy(textString,"\nrootCommand: ~/.blackhome/control/");
   strcat(textString, themeName);
   strcat(textString, ".sh");
   strcat(textString, "\n\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);

/* 3a) create style_r file ./blackhome/control/themeName- for append at end */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/control/");
   strcat(workingFile, themeName);
   strcat(workingFile, "-");
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);
      return 1;
   }
  
   /* add rootCommand to end of the style- file: */
   strcpy(textString,"rootCommand: ~/.blackhome/control/remove.sh ");
   strcat(textString, themeName);
   strcat(textString, " ~ "); /* uninstall needs ~ as param. 2 */
   strcat(textString, imageName);
   strcat(textString, "\n\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);
   

/* 4) create or update ~/.blackhome/styles/stylesmenu
      set new workingFile to ~/.blackhome/styles */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/styles/stylesmenu");
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);   
      return 1;
   }

   strcpy(textString, "[style] (");
   strcat(textString, themeName);
   strcat(textString, ")            {~/.blackhome/styles/");
   strcat(textString, themeName);
   strcat(textString, "}\n");
   strcat(textString, "\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);
   /* updated stylesmenu */
  
   /* create or update removemenu */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/control/removemenu");
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);   
      return 1;
   }

   strcpy(textString, "[style] (");
   strcat(textString, themeName);
   strcat(textString, ")            {~/.blackhome/control/");
   strcat(textString, themeName);
   strcat(textString, "-}\n");
   strcat(textString, "\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);
  /* updated removemenu  - calls a remove script via themeName- */


/* 5) Write out a theme-named copy of ~/.blackhome/menus/menu as default
      edit the menu file to insert current theme name in user styles menu */
   strcpy(inFile, homeDirName);
   strcat(inFile, "/.blackhome/menus/menu");
   strcpy(outFile, homeDirName);
   strcat(outFile, "/.blackhome/menus/");
   strcat(outFile, themeName);
   strcat(outFile, ".menu");

   textString [ 0 ] = 0;
   isExitLine = 0;

   if ((fileHandle2 = fopen((outFile), "w")) == NULL)
      printf("Error: cannot open file %s for writing\n", outFile);
   else
   {  
      if ((fileHandle = fopen(inFile, "r")) != NULL)
      {  
         c = 0;
         lineCount = 0;

         do
         {
            fread(&c, 1, 1, fileHandle);        
            textString[lineCount] = c;

            if (c == 0x0a) /* got current line */
            {
               textString[lineCount+1] = 0x00;
               count = 0;

               if ((textString[0] != '#') && (textString[0] != 0x0a))
               {
                  while (textString[count] != '(')
                     count++;

                  count++; /* advance to 1st char inside bracket */
                  themeCount = 0;
               
                  while (textString[count] != ')') 
                  {
                     embeddedTheme[themeCount] = textString[count];
                     themeCount++;
                     count++;

                  	 if (count >= MAXCHARS)
                     {
                     	printf("overflow: rejecting %s\n", embeddedTheme);
                        return 1;
                     }
                  }

                  embeddedTheme[themeCount] = 0x00;
               }

                  /* check if the bracketed string is 'User styles' */
                  if (strcmp("User styles", embeddedTheme) == 0)
                  { /* count points to ')', write this line, then themeName */
                     count = 0;

                     while (textString[count] != '{')
                     {
                        tempName[count] = textString[count];
                        count++;
                        
                        if (count >= MAXCHARS)
                        {
                        	printf("overflow: rejecting %s\n", tempName);
                           return 1;
                        }
                        
                     }  /* got 1st part of new line */

                     fputs(tempName, fileHandle2);

                     strcpy(tempName, "{");
                     strcat(tempName, themeName);
                     strcat(tempName, "}\n");
                     fputs(tempName, fileHandle2);
                     textString [ 0 ] = 0;
                  }

			      /* check for 'Exit', then write theme-named exit line
			        '[exec] (exit) {./.blackhome/control/themeName-exit.sh} */
                  if (strcmp("Exit", embeddedTheme) == 0 && isExitLine == 0)
                  { 
					 isExitLine = 1;	/* avoid picking up 'Exit' in the new line */
                     strcpy (textString, "  [exit] (Exit) {./.blackhome/control/");
                     strcat (textString, themeName);
                     strcat (textString, "-exit.sh}\n");
				  }

               fputs(textString, fileHandle2);
               textString [ 0 ] = 0;
               lineCount = -1;

            }  /* end got current line */

            lineCount++;
         }  while (!feof(fileHandle));

         fclose(fileHandle);

      }
      else
         printf("Cannot open %s for reading\n", inFile);
   }

   fclose(fileHandle2);

/* 6) Create default theme .Xdefaults-themeName and .Xresources-themeName
      if required (shellFlags 1st char y/n) */

   if (shellFlags[0] == 'y')
   {
      printf("Building with .Xdefaults and .Xresources for theme %s\n", themeName);  
      /* set up themed copy of sysCall .Xdefaults */
      strcpy(inFile, homeDirName);
      strcat(inFile, "/.blackhome/Xdefaults/.Xdefaults-color");
      strcpy(outFile, homeDirName);
      strcat(outFile, "/.blackhome/Xdefaults/.Xdefaults-");
      strcat(outFile, themeName);

      if (filecopy(inFile, outFile) != 0)
      {
         printf("file copying error from %s\nto %s\n", inFile, outFile);
         return 1;
      }

      /* created .Xdefaults-themeName with #include for sysCall .Xdefaults */
      /* there are some global settings which don't work in .Xresources */

      /* now set up themed copy of sysCall .Xresources */
      strcpy(inFile, homeDirName);
      strcat(inFile, "/.blackhome/Xdefaults/.Xresources-color");
      strcpy(outFile, homeDirName);
      strcat(outFile, "/.blackhome/Xdefaults/.Xresources-");
      strcat(outFile, themeName);

      if (filecopy(inFile, outFile) != 0)
      {
         printf("file copying error from %s\nto %s\n", inFile, outFile);
         return 1;
      }

      /* created .Xresources-themeName with #include for sysCall .Xresources */
   }

/* 7) Create default theme .gtkrc-themeName for GTK-based apps */
    /* set up themed copy of gtkrc if required (shellFlags 2nd char y/n)*/
   if (shellFlags[1] == 'y')
   {
      printf("Building with gtkrc for theme %s\n", themeName);
      strcpy(inFile, homeDirName);
      strcat(inFile, "/.blackhome/Xdefaults/gtk/gtkrc-color");
      strcpy(outFile, homeDirName);
      strcat(outFile, "/.blackhome/Xdefaults/gtk/gtkrc-");
      strcat(outFile, themeName);

      fileHandle = fopen(outFile, "a");

      if (fileHandle == NULL)
      {
         printf("fopen error: %ld\n", (long) fileHandle);
         printf ("trying to write %s\n", outFile);   
         return 1;
      }

	  /* insert pixmap-path file */
	  strcpy(textString, "include \".blackhome/Xdefaults/gtk/pixmap-path\"\n");
	  fputs(textString, fileHandle);
      /* insert gtk-pre- and poststyle */
	  strcpy(textString, "include \".blackhome/Xdefaults/gtk/gtk-prestyle\"\n");
      fputs(textString, fileHandle);

      fclose(fileHandle);

      strcpy(textString, "cat ");
      strcat(textString, outFile);
      strcat(textString, " ");
      strcat(textString, inFile);
      strcat(textString, " > ~/.blackhome/Xdefaults/gtk/gtkrc-temp");
      sysCall(textString);

      strcpy(inFile, homeDirName);
      strcat(inFile, "/.blackhome/Xdefaults/gtk/gtkrc-temp");
      fileHandle = fopen(inFile, "a");
		strcpy(textString, "include \".blackhome/Xdefaults/gtk/gtk-poststyle\"\n");
      fputs(textString, fileHandle);
		fclose(fileHandle);

      strcpy(textString, "mv ~/.blackhome/Xdefaults/gtk/gtkrc-temp ");
      strcat(textString, outFile);
      sysCall(textString);
   }

/* 8) Create useful app-resources defaults for the theme
      entries have no leading whitespace, are in pairs, ending with newline
      generates resource files from apps in appDefsList */

   strcpy(workingFile, homeDirName);
   strcat (workingFile, "/.blackhome/config/appDefsList");

   /* find application defaults name pairs */
   /* (appResourceFileName,userResourceFileName) */

   textString [ 0 ] = 0;
   lineCount = 0;
   count = 0;

   if ((fileHandle = fopen(workingFile, "r")) != NULL)
   {
   c = 0;

     do
     {
         fread(&c, 1, 1, fileHandle);
         textString[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if ((textString[0] != '#') && (textString[0] != 0x0a))
            {
               textString[lineCount+1] = 0x00;
               count = 0;

               while(textString[count] != ',')
               {
                  appDefsName[count] = textString[count];
                  count++;
               }

               appDefsName[count] = 0x00;
               count ++;
               themeCount = 0; /* borrow this for a temp counter */

               while(textString[count] != 0x0a)
               {
                  userDefsName[themeCount] = textString[count];
                  count++;
                  themeCount++;
                  
               	if (themeCount >= MAXCHARS)
                  {
                  	printf("overflow: user resource name %s too long\n",
                           	userDefsName);
                     return 1;
                  }
               }

               userDefsName[themeCount] = 0;

               /* now use these to set up app resource files for theme */
               strcpy(inFile, homeDirName);
               strcat(inFile, "/.blackhome/app-defaults/");
               strcat(inFile, appDefsName);
               strcpy(outFile, homeDirName);
               strcat(outFile, "/.blackhome/app-defaults/");
               strcat(outFile, userDefsName);
               strcat(outFile, "-");
               strcat(outFile, themeName);

               if (filecopy(inFile, outFile) != 0)
               {
                  printf("file copying error from %s\nto %s\n", inFile, outFile);
                  return 1;
               }
           }

            lineCount = -1;
         }

         lineCount++;
      }  while (!feof(fileHandle));
  }
  else
  {
     printf("fopen error: %ld\n", (long) fileHandle);
     printf ("trying to access %s\n", workingFile);
     return 1;
  }
 
  fclose(fileHandle);

/* 9a) Write control exit script in ~/.blackhome/control named for the theme
	   This calls 'cleanup.sh' after executing user commands, if any */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/control/");
   strcat(workingFile, themeName);
   strcat(workingFile, "-exit.sh");
   
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to write %s\n", workingFile);   
	  return (1);
   }

   strcpy (textString, "#!/bin/sh\n");
   fputs (textString, fileHandle);
   strcpy (textString, "./.blackhome/control/cleanup.sh\n");
   fputs (textString, fileHandle);
   fclose (fileHandle);
   textString[0] = 0;

/* 9b) Write control script in ~/.blackhome/control named for the theme,
      adding in existing rootCommand (if any) before blackhome's default
	  rootCommand (in case 'none' specified as image) */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/control/");
   strcat(workingFile, themeName);
   strcat(workingFile, ".sh");
   
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);   
   }

   /* Now open userSpec file ro specifying sound command */
   /* eg 'play' */
   /* find uncommented line number 'findLineNum' */

   strcpy(inFile, homeDirName);
   strcat (inFile, "/.blackhome/config/userSpec");
   tempName [ 0 ] = 0;
   lineCount = 0;
   count = 0;
   findLineNum = 1; /* 1st uncommented line is the sound player command */

   if ((fileHandle2 = fopen(inFile, "r")) != NULL)
   {
      while (!feof(fileHandle2))
      {
        fread(&c, 1, 1, fileHandle2);

         if (c != 0x0a) /* skip ending newline */
         {
            tempName[lineCount] = c;
            lineCount++;

            if (lineCount >= MAXCHARS)
            {
            	printf("overflow: reading %s\n, rejecting", tempName);
               return 1;
            }
         }
         else
         {  /* found end of line code */
            tempName[lineCount] = 0x00;
            count++;
               
            if (tempName[0] == '#')
            {
               tempName [ 0 ] = 0;
               lineCount = 0;
               count--;
            }

            if (count == findLineNum)
               strcpy(textString, tempName);
            else
            {
               tempName [ 0 ] = 0;
               lineCount = 0;
             }
         }
      } /* end of file */

      strcpy(tempName, textString);
      fclose(fileHandle2); /* close userSpec file */
   }
   else
   {
      printf("User-defined external program definitions file 'userSpec'\n");
      printf("not found in ~/.blackhome/config/ !\n");
   }

   strcpy(textString, "#!/bin/sh\n");

   /* run the preprocess script for actions common to all themes */
   strcat(textString, "~/.blackhome/control/preprocess.sh\n");

   /* link default sound, menus to new style */ 
   if ( strlen ( tempName ) == 0 )
      strcat(textString, "# no sound file player specified\n");
   else
      strcat(textString, tempName);

   if (strcmp("play", tempName) == 0)
   {
      strcat(textString, " ~/.blackhome/app-defaults/sounds/");
      strcat(textString, themeName); /* default .wav named for theme */
      strcat(textString, ".wav &\n");
   }
   else
   {
      if (strcmp("playmidi", tempName) == 0)
         strcat(textString, " ~/.blackhome/app-defaults/sounds/default.mid &\n");
      else
         strcat(textString, "# No midi play command defined in userSpec\n");
   }

   strcat(textString, "ln -sf ~/.blackhome/menus/");
   strcat(textString, themeName);
   strcat(textString, ".menu ");
   strcat(textString, "~/.blackhome/menus/default.menu\n");
   strcat(textString, "\0");
   fputs(textString, fileHandle);

   strcpy(textString, "ln -sf ~/.blackhome/menus/menu_images_");
   strcat(textString, themeName);
   strcat(textString, " ~/.blackhome/menus/menu_images_default\n");
   strcat(textString, "\0");
   fputs(textString, fileHandle);

   strcpy(textString, "ln -sf ~/.blackhome/menus/menu_pixmaps_");
   strcat(textString, themeName);
   strcat(textString, " ~/.blackhome/menus/menu_pixmaps_default\n");
   strcat(textString, "\0");
   fputs(textString, fileHandle);

   /* write 'current_theme' file in .blackhome/control */
   strcpy(textString, "echo ");
   strcat(textString, themeName);
   strcat(textString, " > .blackhome/control/current_theme\n#\n");
   fputs(textString, fileHandle);
   
   /* symlink defaults to .Xdefaults and .Xresources */
   /* dependent on ShellFlags[0] status y/n */
   if (shellFlags[0] == 'y')
   {
      strcpy(textString, "ln -sf ~/.blackhome/Xdefaults/.Xdefaults-");
      strcat(textString, themeName);
      strcat(textString, " ~/.Xdefaults\n");
      strcat(textString, "xrdb -load ~/.Xdefaults\n");
      strcat(textString, "\0");
      fputs(textString, fileHandle);

      strcpy(textString, "ln -sf ~/.blackhome/Xdefaults/.Xresources-");
      strcat(textString, themeName);
      strcat(textString, " ~/.Xresources\n");
      strcat(textString, "xrdb -merge ~/.Xresources\n");
      strcat(textString, "\0");
      fputs(textString, fileHandle);
   }

   /* symlink defaults to .gtkrc */
   strcpy(textString, "ln -sf ~/.blackhome/Xdefaults/gtk/gtkrc-");
   strcat(textString, themeName);
   strcat(textString, " ~/.gtkrc.mine\n#\n");
   fputs(textString, fileHandle);

   /* Now we set up links to useful application defaults */
   /* read apps list from ~/.blackhome/config/linkDefsList */
   strcpy(inFile, homeDirName);
   strcat (inFile, "/.blackhome/config/linkDefsList");
   textString[0] = 0;
   lineCount = 0;
   count = 0;

   if ((fileHandle2 = fopen(inFile, "r")) != NULL)
   {
      c = 0;

      do
      {
         fread(&c, 1, 1, fileHandle2);
         textString[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if ((textString[0] != '#') && (textString[0] != 0x0a))
            {
               textString[lineCount+1] = 0x00;
               count = 0;

               while(textString[count] != ',')
               {
                  appDefsName[count] = textString[count];
                  count++;
                  
               	if (count >= MAXCHARS)
                  {
                  	printf("overflow: %s, rejecting\n", appDefsName);
                     return 1;
                  }
                  
               }

               appDefsName[count] = 0x00;
               count ++;
               themeCount = 0; /* borrow this for a temp counter */

               while(textString[count] != 0x0a)
               {
                  userDefsName[themeCount] = textString[count];
                  count++;
                  themeCount++;
                  
               	if (themeCount >= MAXCHARS)
                  {
                  	printf("overflow: %s, rejecting\n", userDefsName);
                     return 1;
                  }
                  
               }

               userDefsName[themeCount] = 0;

               /* now use these to set up defaults links in ~ */
               strcpy(textString, "ln -sf ~/.blackhome/app-defaults/");
               strcat(textString, appDefsName);
               strcat(textString, "-");
               strcat(textString, themeName);
               strcat(textString, " ~/");
               strcat(textString, userDefsName);
               strcat(textString, "\n\0");
               fputs(textString, fileHandle);
            }

            lineCount = -1;
         }

         lineCount++;
      } while (!feof(fileHandle2));
   }
   else
   {
      printf("fopen error: %ld\n", (long) fileHandle2);
      printf ("trying to access %s\n", inFile);
      return 1;
   }

   fclose(fileHandle2);
   
   if ( strlen ( styleRC ) != 0 )	/* existing rootCommand from style file */
   {
      fputs("#\n", fileHandle);	/* separate from symlink commands for clarity */

		if (styleRC[0] == '#')
			styleRC[0] = ' ';

      fputs(styleRC, fileHandle);
   }

   /* open user file ro specifying image viewer command
   eg 'xv -root -quit', or 'display -geometry 800x600', or...
   find uncommented line number 'findLineNum'. */
 
   if (strcmp(imageName, "none") != 0)
   {
      strcpy(inFile, homeDirName);
      strcat (inFile, "/.blackhome/config/userSpec");
      tempName [ 0 ] = 0;
      lineCount = 0;
      count = 0;
      findLineNum = 2; /* 2nd uncommented line is the image viewer command */

      if ((fileHandle2 = fopen(inFile, "r")) != NULL)
      {
    	  while (!feof(fileHandle2))
    	  {
        	fread(&c, 1, 1, fileHandle2);

        	 if (c != 0x0a) /* skip ending newline */
        	 {
            	tempName[lineCount] = c;
            	lineCount++;

            	if (lineCount >= MAXCHARS)
            	{
            		printf("overflow: %s, rejecting\n", tempName);
            	   return 1;
            	}
        	 }
        	 else
        	 {  /* found end of line code */
            	tempName[lineCount] = 0x00;
            	count++;

            	if (tempName[0] == '#')
            	{
            	   tempName [ 0 ] = 0;
            	   lineCount = 0;
            	   count--;
            	}

            	if (count == findLineNum)
            	   strcpy(textString, tempName);
            	else
            	{
            	   tempName [ 0 ] = 0;
            	   lineCount = 0;
         	}
          }
      } /* end of file */

      strcpy(tempName, textString);
      fclose(fileHandle2);
      }
      else
      {
    	  printf("User-defined external program definitions file 'userSpec'\n");
    	  printf("not found in ~/.blackhome/config/ !\n");
      }

      strcpy(textString, "#\n");
      strcat(textString, tempName); /* insert user-defined viewer */
      strcat(textString, " ~/.blackhome/styles/");
      strcat(textString, imageName);
      strcat(textString, "\n");
      strcat(textString, "\0");
      fputs(textString, fileHandle);  

      fclose(fileHandle);
   }
   
   strcpy(textString, "\nDefault theme application resource files have been created as\n");
   strcat(textString, homeDirName);
   strcat(textString, "/.blackhome/app-defaults/applicationName-");
   strcat(textString, themeName);
   strcat(textString, "\nand theme global resources as\n");
   strcat(textString, homeDirName);
   strcat(textString, "/.blackhome/Xdefaults/.Xdefaults-");
   strcat(textString, themeName);
   strcat(textString, "\n");
   strcat(textString, homeDirName);
   strcat(textString, ".blackhome/Xdefaults/.Xresources-");
   strcat(textString, themeName);
   strcat(textString, "\n");
   strcat(textString, homeDirName);
   strcat(textString, "/blackhome/Xdefaults/gtk/gtkrc-");
   strcat(textString, themeName);
   strcat(textString, "\nEdit these to taste.\n\n");
   printf(textString); 
   printf("Theme %s setup completed.\n", themeName);
   printf("Start (or restart) Blackbox\n\n");
   return 0;
   /* all ok */
}

int filecopy(char *in, char *out)
{
   FILE *fileHandle, *fileHandle2;
   char c;

   fileHandle = fopen(in, "r");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to read %s\n", in);   
      return 1;
   }

   fileHandle2 = fopen(out, "a");

   if (fileHandle2 == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle2);
      printf ("trying to write %s\n", out);   
      return 1;
   }

   /* now copy the file */ 
   do
   {
      c = fgetc(fileHandle);

      if (!feof(fileHandle))
         fputc(c, fileHandle2);
   } while (!feof(fileHandle));

   fputs("\0", fileHandle2);
      
   fclose(fileHandle2);
   fclose(fileHandle);
   return 0;
}

int uninstall(char *removeTheme, char *topDir, char *removeImage)
{
   FILE *in, *out;
   char themeName[MAXCHARS];
   char imageName[MAXCHARS];
   char embeddedTheme[MAXCHARS];
   char tempName[MAXCHARS];
   char homeDirName[MAXCHARS];
   char workingFile[MAXCHARS];
   char outFile[MAXCHARS];
   char line[16*MAXCHARS]; /* set up line to write to file */
   char c;
   unsigned int count;
   unsigned int lineCount;
   unsigned int themeCount; /* count chars in theme name */

   sprintf(themeName, "%s", removeTheme);
   sprintf(homeDirName, "%s", topDir);
   sprintf(imageName, "%s", removeImage);

   strcpy(workingFile, homeDirName);
   strcat(workingFile,"/.blackhome/control/removemenu");
   strcpy(outFile, workingFile);
   strcat(outFile, ".tmp");

   if ((out = fopen(outFile, "w")) == NULL)
   {
      printf("error attempting to write %s\n", outFile);
      exit(1);
   }

   if ((in = fopen(workingFile, "r")) == NULL)
   {
      printf("error attempting to read %s\n", workingFile);
      exit(1);
   }
   else
   {
      c = 0;
      lineCount = 0;

      do
      {
         fread(&c, 1, 1, in);
         line[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if (line[0] == '#') /* skip if comment line */
            {
               strcat(line, "\n");
               fwrite(line, 1, (lineCount + 1), out);
            }
            else
            { 
               if (line[0] == '[') /* theme line start */
               {
                  count = 0;

                  while (line[count] != '(')
                     count++; /* found start of theme name entry */

                  count++;
                  themeCount = 0;
               
                  while (line[count] != ')')
                  {
                     embeddedTheme[themeCount] = line[count];
                     themeCount++;
                     count++;
                  }

                  strcpy(tempName,themeName);
                  embeddedTheme[themeCount] = 0x00;

                  if (strcmp(embeddedTheme, tempName) != 0)
                  {
                     strcat(line, "\n");
                     fwrite(line, 1, (lineCount + 1), out);
                  }
               }
            }

            strcpy(line, "");
            lineCount = -1;
         }

         lineCount++;
      } while (!feof(in));
   }

   fclose(in);
   fclose(out);

   /* now edit stylesmenu */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/styles/stylesmenu");
   strcpy(outFile, homeDirName);
   strcat(outFile, "/.blackhome/control/stylesmenu.tmp"); /* moved to 'styles'
														     dir by remove.sh */

   if ((out = fopen(outFile, "w")) == NULL)
   {
      printf("error attempting to write %s\n", outFile);
      exit(1);
   }

   if ((in = fopen(workingFile, "r")) == NULL)
   {
      printf("error attempting to read %s\n", workingFile);
      exit(1);
   }
   else
   {
      c = 0;
      lineCount = 0;
      embeddedTheme [ 0 ] = 0;
      line [ 0 ] = 0;
      do
      {
         fread(&c, 1, 1, in);
         line[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if (line[0] == '#') /* skip if comment line */
            {
               strcat(line, "\n");
               fwrite(line, 1, (lineCount + 1), out);
            }
            else
            { 
               if (line[0] == '[') /* theme line start */
               {
                  count = 0;

                  while (line[count] != '(')
                  count++; /* found start of theme name entry */

                  count++;
                  themeCount = 0;
               
                  while (line[count] != ')')
                  {
                     embeddedTheme[themeCount] = line[count];
                     themeCount++;
                     count++;
                  }

                  strcpy(tempName,themeName);
                  embeddedTheme[themeCount] = 0;

                  if (strcmp(embeddedTheme, tempName) != 0)
                  {
                     strcat(line, "\n");
                     fwrite(line, 1, (lineCount + 1), out);
                  }
               }
            }

            line [ 0 ] = 0;
            lineCount = -1;
         }

         lineCount++;
      }  while (!feof(in));
   }

   fclose(in);
   fclose(out);

/* remove images. pixmaps, sounds associated with theme */
   strncpy(line, "if [ -d ~/.blackhome/app-defaults/images/", sizeof (line));
   strncat(line, themeName, sizeof (line)-strlen(line)-1);
   strncat(line, " ]; then \nrm -rf ~/.blackhome/app-defaults/images/",sizeof (line)-strlen(line)-1);
   strncat(line, themeName, sizeof (line)-strlen(line)-1);
   strncat(line, "\nfi", sizeof (line)-strlen(line)-1);
   line[strlen(line)] = 0;
   sysCall(line);

   strncpy(line, "if [ -d ~/.blackhome/app-defaults/pixmaps/", sizeof (line));
   strncat(line, themeName, sizeof (line)-strlen(line)-1);
   strncat(line, " ]; then \nrm -rf ~/.blackhome/app-defaults/pixmaps/",sizeof (line)-strlen(line)-1);
   strncat(line, themeName, sizeof (line)-strlen(line)-1);
   strncat(line, "\nfi", sizeof (line)-strlen(line)-1);
   line[strlen(line)] = 0;
   sysCall(line);

   strncpy(line, "if [ -r ~/.blackhome/app-defaults/sounds/", sizeof (line));
   strncat(line, themeName, sizeof (line)-strlen(line)-1);
   strncat(line, "* ]; then \nrm ~/.blackhome/app-defaults/sounds/",sizeof (line)-strlen(line)-1);
   strncat(line, themeName, sizeof (line)-strlen(line)-1);
   strncat(line, "*\nfi", sizeof (line)-strlen(line)-1);
   line[strlen(line)] = 0;
   sysCall(line);

   /* remove theme background image, if any */
   strncpy(line, "if [ -r ~/.blackhome/styles/", sizeof (line));
   strncat(line, imageName, sizeof (line)-strlen(line)-1);
   strncat(line, " ]; then \nrm ~/.blackhome/styles/", sizeof(line)-strlen(line)-1);
   strncat(line, imageName, sizeof(line)-strlen(line)-1);
   strncat(line, "\nfi", sizeof(line)-strlen(line)-1);
   line[strlen(line)] = 0;
   sysCall(line);

   return 0;
}


int removeLinks(char *homeDirName)
{
   char c;
   char inFile[MAXCHARS];
   FILE *fileHandle;
   char textString[MAXCHARS];
   int lineCount;
   char linkDefsName[MAXCHARS];
   unsigned int src, dst;
   
   /* first remove non-app-specific links */
   sysCall("rm ~/.Xdefaults 2&> /dev/null");
   sysCall("rm ~/.Xresources 2&> /dev/null");
   
   /* read link list from ~/.blackhome/config/linkDefsList */
   strcpy(inFile, homeDirName);
   strcat (inFile, "/.blackhome/config/linkDefsList");
   textString [ 0 ] = 0;
   lineCount = 0;
       
   if ((fileHandle = fopen(inFile, "r")) != NULL)
   {
      c = 0;
    
      do
      {
         fread(&c, 1, 1, fileHandle);
         textString[lineCount] = c;
      
         if (c == 0x0a) /* got current line */
         {
            if ((textString[0] != '#') && (textString[0] != 0x0a))
            {
               textString[lineCount+1] = 0x00;
               src = 0;
               dst = 0;

		       while(textString[src] != ',')
	    	      src++;
               /* ignore the userDefsName */

               src++;

               while(textString[src] != 0x0a)
               { /* read the link name itself */
               	linkDefsName[dst] = textString[src];
                  src++;
                  dst++;
               } 
               /* now use this to remove link in ~ */
		
               strcpy(textString, "rm -f ~/");
               strcat(textString, linkDefsName);
		       sysCall(textString);
               src = 0;

               while (src < MAXCHARS)
               {
                  textString[src] = 0;
                  linkDefsName[src] = 0;
                  src ++;
               }
            }
   
            lineCount = -1;
         }
  
         lineCount++;
      } while (!feof(fileHandle));
   }
   else 
   {   
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", inFile);
      return 1; 
   }     
            
   fclose(fileHandle);

   return 0;
}


int sysCall ( const char *command )
{  /* code ripped from GNU 'system' man page */
   int pid, status;
                                
   if ( command == 0 )
       return 1;

   pid = fork();

   if ( pid == -1 )
       return -1;

   if ( pid == 0 )
   {
       char *argv[4];
       argv[0] = "sh";
       argv[1] = "-c";
       argv[2] = ( char * ) command;
       argv[3] = 0;
       execve( "/bin/sh", argv, environ );
       exit ( 127 );
   }

   do
   {
       if ( waitpid ( pid, &status, 0 ) == -1 )
       {
          if ( errno != EINTR )
             return -1;
       }
       else
          return status;
   } while ( 1 );
}
