/*
   installImage - install and configure blackbox theme pixmaps and images.
   Copyright (C) 1999 Will Eighteen: will@opaline.demon.co.uk

   This program assumes that the theme image files are in the
   ~/.blackhome/app-defaults/images/themeName
   directory, and that the theme pixmap files are in the
   ~/.blackhome/app-defaults/pixmaps/themeName
   directory.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    See .blackhome/COPYING for full details.
*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>

#define MAXCHARS 256

int uninst(char *theme, char *image, char *topDir);

int main(int argc, char * argv[])
{

   char themeName[MAXCHARS];
   char imageName[MAXCHARS];
   char homeDirName[MAXCHARS];
   char workingFile[3*MAXCHARS];
   char tempName[MAXCHARS];
   char embeddedImage[MAXCHARS];
   char c;
   char imageTypeDir[10];
   char textString[16*MAXCHARS]; /* set up line to write to file */
   char rmImage[1];
   unsigned int pixmap; /* flag is .xpm or some other image format */
   unsigned int lineCount;
   unsigned int count;
   unsigned int themeCount;
   FILE *fileHandle;

   if (argc < 4)
   {
      printf("blackhome-0.2.2: Will 18 1999\n");
      printf("Use ~/.blackhome/config/install image themeName imageName\n");
      printf("\nEnsure the theme image and pixmap files are in the\n");
      printf("~/.blackhome/app-defaults/images/themeName\n");
      printf("and\n");
      printf("~/.blackhome/app-defaults/pixmaps/themeName\n");
      printf("directories respectively\n");
      return 1;
   }

   if (strlen(argv[1]) < MAXCHARS-1 )
      sprintf(themeName, "%s", argv[1]);
   else
   {
      printf("theme name %s more than %u chars, rejecting...\n",
            	argv[1], MAXCHARS-1);
      return 1;
   }

   if (strlen(argv[2]) < MAXCHARS-1 )
      sprintf(imageName, "%s", argv[2]);
   else
   {
      printf("image name %s more than %u chars, rejecting...\n",
            	argv[2], MAXCHARS-1);
      return 1;
   }

   if (strlen(argv[3]) < MAXCHARS-1 )
      sprintf(homeDirName, "%s", argv[3]);
   else
   {
      printf("home dir name %s more than %u chars, rejecting...\n",
            	argv[3], MAXCHARS-1);
      return 1;
   }

   if (strstr(imageName, ".xpm") == NULL)
   {
      pixmap = 0;	/* not a pixmap, set a flag */
      strcpy(imageTypeDir, "images/");
   }
   else
   {
      pixmap = 1;
      strcpy(imageTypeDir, "pixmaps/");
   }

   if (argc == 5) /* extra argument is 'u' supplied by shell */
   { 
      if (strlen(argv[4]) > 1)
      {
         printf("error: %s should be character u!\n!", argv[4]);
         return 1;
      }  
      else
         sprintf(rmImage, "%s", argv[4]);

      if (strstr(rmImage, "u") == 0)
      {
         printf("error uninstalling %s from %s\n", imageName, themeName);
         printf("use installImage themeName imageName pathToHomeDirectory u\n");
         return 1;
      }

      if (uninst(themeName, imageName, homeDirName) != 0)
      {
         printf("failed uninstalling %s from %s\n", imageName, themeName);
         return 1;
      }
      else
      {
         printf("uninstalled %s from %s\n", imageName, themeName);
         return 0;
      }
   }

   strcpy(workingFile, homeDirName);
   strcat(workingFile,"/.blackhome/app-defaults/");
   strcat(workingFile, imageTypeDir);     
   strcat(workingFile, themeName);
   strcat(workingFile, "/removemenu");

   /* check for imageName in removemenu - if present, exit install */
   if ((fileHandle = fopen(workingFile, "r")) != NULL)
   {  /* some user styles are already installed */
      c = 0;
      lineCount = 0;

      do
      {
         fread(&c, 1, 1, fileHandle);        
         textString[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if (textString[0] == '[') /* theme line start */
            {
               count = 0;

               while (textString[count] != '(')
               count++; /* found start of theme name entry */

               count++;
               themeCount = 0;
               
               while (textString[count] != ')') /* do till ) */
               {
                  embeddedImage[themeCount] = textString[count];
                  themeCount++;
                  count++;
               }

               embeddedImage[themeCount] = 0;
               strcpy(tempName, imageName);

               if (strcmp(embeddedImage, tempName) == 0)
               {
                  if (pixmap == 1)
                     printf("pixmap %s is already installed!\n", imageName);
                  else
                     printf("image %s is already installed\n", imageName);

                  printf("Remove this first or change the name.\n");
                  return 1;
               }
            }

            textString [ 0 ] = 0;
            lineCount = -1;
         }  /* end got current line */

         lineCount++;
      }  while (!feof(fileHandle));
   
      fclose(fileHandle);
   }
   else
   {
      printf("file %s not created yet, sorting it...\n", workingFile);
   }

   printf("Setting up %s for %s\n", imageName, themeName);

   strcpy(workingFile, homeDirName);
   strcat(workingFile,"/.blackhome/app-defaults/");
   strcat(workingFile, imageTypeDir);     
   strcat(workingFile, themeName);
   strcat(workingFile, "/");
   strcat(workingFile, imageName);
   strcat(workingFile, ".sh");

/* 1) create shell script linking image to default */
   fileHandle = fopen(workingFile, "a");

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);
      return 1;
   }
 
   strcpy(textString,"#!/bin/sh\nln -sf ~/.blackhome/app-defaults/");   
   strcat(textString, imageTypeDir);
   strcat(textString, themeName);
   strcat(textString, "/");
   strcat(textString, imageName);
   strcat(textString, " ~/.blackhome/menus/default");

   if (!pixmap)
      strcat(textString, ".image\n");
   else
      strcat(textString, ".xpm\n");

   strcat(textString, "\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);
   /* created image or pixmap shell script */  
   

/* 2) create ~/.blackhome/menus pixmaps or images submenu if needed*/
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/menus/");

   strcat(workingFile, "menu_");
   
   if (!pixmap)
      strcat(workingFile, "images_");
   else
      strcat(workingFile, "pixmaps_");
     
   strcat(workingFile, themeName);   

   if (fopen(workingFile, "r") == NULL) /* file doesn't exist, we hope */
   {
      /* create new file */
      fileHandle = fopen(workingFile, "a");

      if (fileHandle == NULL)
      {
         printf("fopen error: %ld\n", (long) fileHandle);
         printf ("trying to access %s\n", workingFile);   
         return 1;
      }

      strcpy(textString, "[submenu] (Remove ");

      if (!pixmap)
         strcat(textString, "image)");
      else
         strcat(textString, "pixmap)");

      strcat(textString, "\n[include] (~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/removemenu)\n[end]\n");
      strcat(textString, "\0");
      fputs(textString, fileHandle);    
      fclose(fileHandle);
   }


/* 2a) create ~/.blackhome/app-defaults/imageType/themeName removemenu */
   /* if needed*/
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/app-defaults/");
   strcat(workingFile, imageTypeDir);  
   strcat(workingFile, themeName);
   strcat(workingFile, "/removemenu");  

   if (fopen(workingFile, "r") == NULL) 
   {
      /* create new file */
      fileHandle = fopen(workingFile, "a");

      if (fileHandle == NULL)
      {
         printf("fopen error: %ld\n", (long) fileHandle);
         printf ("trying to access %s\n", workingFile);   
         return 1;
      }
      
      fclose(fileHandle);
   }

/* 2b) create ~/.blackhome/app-defaults/imageType/themeName */
   /*  imageName-.sh */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/app-defaults/");
   strcat(workingFile, imageTypeDir);
   strcat(workingFile, themeName);
   strcat(workingFile, "/");
   strcat(workingFile, imageName);
   strcat(workingFile, "-.sh");

   if (fopen(workingFile, "r") == NULL) 
   {
      /* create new file */
      fileHandle = fopen(workingFile, "a");

      if (fileHandle == NULL)
      {
         printf("fopen error: %ld\n", (long) fileHandle);
         printf ("trying to access %s\n", workingFile);   
         return 1;
      }

      /* set up remove script for image */
      strcpy(textString, "#!/bin/sh\n");
      strcat(textString, "rm ~/.blackhome/menus/menu_");

      if (!pixmap)
         strcat(textString, "images");
      else
         strcat(textString, "pixmaps");

      strcat(textString, "_default\n");

      strcat(textString, "rm ~/.blackhome/menus/default.");

      if (!pixmap)
         strcat(textString, "image\n");
      else
         strcat(textString, "pixmap\n");

      /* run installImage 'u' */
      strcat(textString, "~/.blackhome/config/installImage $1 $2 $3 u\n");

      /* delete removemenu */
      strcat(textString, "rm ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/removemenu\n");

      /* delete menu_images_themeName */
      strcat(textString, "rm ~/.blackhome/menus/menu_");

      if (!pixmap)
         strcat(textString, "images_");
      else
         strcat(textString, "pixmaps_");

      strcat(textString, themeName);

      /* move newly rebuilt menus and delete .tmp files */
      strcat(textString, "\ncp ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/removemenu.tmp ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/");
      strcat(textString, "removemenu\n");
      strcat(textString, "rm ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/removemenu.tmp\n");

      strcat(textString, "cp ~/.blackhome/menus/menu_");

      if (!pixmap)
         strcat(textString, "images_");
      else
         strcat(textString, "pixmaps_");

      strcat(textString, themeName);
      strcat(textString, ".tmp ~/.blackhome/menus/menu_");

      if (!pixmap)
         strcat(textString, "images_");
      else
         strcat(textString, "pixmaps_");
 
      strcat(textString, themeName);
      strcat(textString, "\nrm ~/.blackhome/menus/menu_");

      if (!pixmap)
         strcat(textString, "images_");
      else
         strcat(textString, "pixmaps_");

      strcat(textString, themeName);
      strcat(textString, ".tmp\n");

       /* remove imageName.sh */
      strcat(textString, "rm ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/");
      strcat(textString, imageName);
      strcat(textString, ".sh\n");

      /* remove image */
      strcat(textString, "rm ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/");
      strcat(textString, imageName);
      strcat(textString, "\n");
  
      /* finally, remove the remove script */
      strcat(textString, "rm ~/.blackhome/app-defaults/");
      strcat(textString, imageTypeDir);
      strcat(textString, themeName);
      strcat(textString, "/");
      strcat(textString, imageName);
      strcat(textString, "-.sh\n");
      strcat(textString, "\0");
      fputs(textString, fileHandle);
      fclose(fileHandle);
   }
   
/* 3) Update the menu_images_themeName or menu_pixmaps_themeName file */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/menus/");

   strcat(workingFile, "menu_");
   
   if (!pixmap)
      strcat(workingFile, "images_");
   else
      strcat(workingFile, "pixmaps_");
     
   strcat(workingFile, themeName);   

   fileHandle = fopen(workingFile, "a"); /* re-open it */

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);   
      return 1;
   }
   
   strcpy(textString, "[exec] (");
   strcat(textString, imageName);
   strcat(textString, ") ");
   strcat(textString, "{~/.blackhome/app-defaults/");
   strcat(textString, imageTypeDir);
   strcat(textString, themeName);
   strcat(textString, "/");
   strcat(textString, imageName); /* including .xpm if present, sigh */   
   strcat(textString, ".sh}\n");   
   strcat(textString, "\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);
   /* updated images or pixmaps menu for theme */  

/* 3a) Update .blackhome/app-defaults/images\pixmaps/themeName/removemenu */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/app-defaults/");
   strcat(workingFile, imageTypeDir);
   strcat(workingFile, themeName);
   strcat(workingFile, "/removemenu");
 
   fileHandle = fopen(workingFile, "a"); /* re-open it */

   if (fileHandle == NULL)
   {
      printf("fopen error: %ld\n", (long) fileHandle);
      printf ("trying to access %s\n", workingFile);   
      return 1;
   }
   
   strcpy(textString, "[exec] (");
   strcat(textString, imageName);
   strcat(textString, ") ");
   strcat(textString, "{~/.blackhome/app-defaults/");
   strcat(textString, imageTypeDir);
   strcat(textString, themeName);
   strcat(textString, "/");
   strcat(textString, imageName); 
   strcat(textString, "-.sh ");
   strcat(textString, themeName);
   strcat(textString, " ");
   strcat(textString, imageName);
   strcat(textString, " ~");
   strcat(textString, "}\n");
   strcat(textString, "\0");
   fputs(textString, fileHandle);
   fclose(fileHandle);
   /* updated images or pixmaps removemenu for theme */

   printf("%s setup for theme %s completed.\n\n", imageName, themeName);
   strcpy(textString, "To use the new ");

   if (!pixmap)
      strcat(textString, "image");
   else
      strcat(textString, "pixmap");

   strcat(textString," select it first from the ");  

   if (!pixmap)
      strcat(textString, "images menu.");
   else
      strcat(textString, "pixmaps menu.");

   printf("%s\n", textString);
   return 0;
   /* all ok */
}

int uninst(char *theme, char *image, char *topDir)
{
   FILE *in, *out;
   int pixmap;
   char themeName[MAXCHARS];
   char imageName[MAXCHARS];
   char homeDirName[MAXCHARS];
   char embeddedImage[MAXCHARS];
   char workingFile[MAXCHARS];
   char outFile[MAXCHARS];
   char line[16*MAXCHARS]; /* set up line to write to file */
   char c;
   unsigned int count;
   int lineCount;
   unsigned int themeCount;

/* update .blackhome/menus/menu_images_themename or */
/* .blackhome/menus/menu_pixmaps_themeName */

   sprintf(themeName, "%s", theme);
   sprintf(imageName, "%s", image);
   sprintf(homeDirName, "%s", topDir);

   if (strstr(imageName, ".xpm") == NULL)
      pixmap = 0;	/* not a pixmap, set a flag */
   else
      pixmap = 1;

   strcpy(workingFile, homeDirName);
   strcat(workingFile,"/.blackhome/app-defaults/");

   if (!pixmap)
      strcat(workingFile, "images/");
   else
      strcat(workingFile, "pixmaps/");
 
   strcat(workingFile, themeName);
   strcat(workingFile, "/removemenu");

   strcpy(outFile, workingFile);
   strcat(outFile, ".tmp");

   if ((out = fopen(outFile, "w")) == NULL)
   {
      printf("error attempting to write %s\n", outFile);
      _exit(1);
   }

   if ((in = fopen(workingFile, "r")) == NULL)
   {
      printf("error attempting to read %s\n", workingFile);
      _exit(1);
   }
   else
   {
      c = 0;
      lineCount = 0;

      do
      {
         fread(&c, 1, 1, in);
         line[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if (line[0] == '[') /* theme line starts with [ */
            {
               count = 0;

               while (line[count] != '(')
               count++;

               count++;
               themeCount = 0;
               
               while (line[count] != ')') /* do till ) */
               {
                  embeddedImage[themeCount] = line[count];
                  themeCount++;
                  count++;
               }

               embeddedImage[themeCount] = 0;

               if (strcmp(embeddedImage, imageName) != 0)
               {
                  strcat(line, "\n");
                  fwrite(line, 1, (lineCount + 1), out);
               }
            }

            line [ 0 ] = 0;
            lineCount = -1;
         }

         lineCount++;
      } while (!feof(in));
   }

   fclose(in);
   fclose(out);

   /* now edit menu_imageType_themeName */
   strcpy(workingFile, homeDirName);
   strcat(workingFile, "/.blackhome/menus/menu_");

   if (!pixmap)
      strcat(workingFile, "images_");
   else
      strcat(workingFile, "pixmaps_");
 
   strcat(workingFile, themeName);

   strcpy(outFile, homeDirName);
   strcat(outFile, "/.blackhome/menus/menu_");

   if (!pixmap)
      strcat(outFile, "images_");
   else
      strcat(outFile, "pixmaps_");
 
   strcat(outFile, themeName);
   strcat(outFile, ".tmp");

   if ((out = fopen(outFile, "w")) == NULL)
   {
      printf("error attempting to write %s\n", outFile);
      _exit(1);
   }

   if ((in = fopen(workingFile, "r")) == NULL)
   {
      printf("error attempting to read %s\n", workingFile);
      _exit(1);
   }
   else
   {
      c = 0;
      count = 0;
      embeddedImage [ 0 ] = 0;
      line [ 0 ] = 0;

     /* skip 3 lines - [include] for submenu... */

      while(count < 3)
      {
         fread(&c, 1, 1, in);
         fwrite(&c, 1, 1, out);
 
          if (c == 0x0a) /* end of line */
             count++; /* line counter */
      }

      c = 0;
      lineCount = 0;

      do
      {
         fread(&c, 1, 1, in);
         line[lineCount] = c;

         if (c == 0x0a) /* got current line */
         {
            if (line[0] == '[') /* theme line start */
            {
               count = 0;

               while (line[count] != '(')
               count++;

               count++;
               themeCount = 0;
               
               while (line[count] != ')')
               {
                  embeddedImage[themeCount] = line[count];
                  themeCount++;
                  count++;
               }

               embeddedImage[themeCount] = 0;

               if (strcmp(embeddedImage, imageName) != 0)
               {
                  strcat(line, "\n");
                  fwrite(line, 1, (lineCount + 1), out);
               }
            }

            line [ 0 ] = 0;
            lineCount = -1;
         }

         lineCount++;
      }  while (!feof(in));
   }

   fclose(in);
   fclose(out);
   return 0;
}
