/* 
   ResourceFileEditor.h

   Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _RFE_H_
#define _RFE_H_

#include <errno.h>
#include <ctype.h>

#include "Global.h"
#include "colourSpecConvert/ColourSpecConvert.h"
#include "menu/Menu.h"
#include "text/Text.h"
#include "List.h"

#include <X11/StringDefs.h>		/* Standard Name-String definitions */
#include <X11/Intrinsic.h>

#include <X11/Xaw3d/Form.h>
#include <X11/Xaw3d/Command.h>


typedef struct _RfeResources 
{
	int topLabelWidth;
	int maxMenuItems;
    String rgbTxtDir;
	String XdefaultsDir;
    String appResDir;
    String stylesDir;
	String menusDir;
	String resourceDir;
	String currentThemeSpecifier;
} rfeResources;


class ResourceFileEditor
{
	public:
		ResourceFileEditor( char *, XtPointer, const char *, Widget, XtAppContext );
		~ResourceFileEditor();
		
	private:
		XtAppContext _app;
		Widget _button, _button2;
		Widget _label;
		Widget _form;
		Widget _popup;
		char currentThemeName [ MAXCHARS ];
		char currentEditLine [ EDITBUFSIZE ];
		int sysCall ( const char * );

	protected:
		struct App {
						char * name;
						char * resourceName;
					};

		struct Ops {
						char * fileName;
						char * resourceLabel;	
						char * returnedResource;
						char * specifiedResource;
						unsigned int * resourceType;	// none, app, colour, widget
					};
		
		struct RfeObjectData {
								List < App > apps;
								List < char * > themes;
								List < char * > aliases;
								List < char * > selApps;	// selected apps
								List < char * > selThemes;	// selected themes
								List < char * > targets;	// selected files
								List < Ops > opOn;			// operate on...
								List < char * > replace;	// new lamps for old
								float redMin;
								float redMax;
								float greenMin;
								float greenMax;
								float blueMin;
								float blueMax;
								float globalColourBound;
							};
							
		RfeObjectData rfeObjectData;
		rfeResources * myResources;

		char _homeDir [ MAXCHARS ];
		char _gnomeColourFormatString [ MAXCHARS ];
		int _menuItem;
		int _maxMenuItems;
		int _maxThemeMenuItems;
		int _maxAppMenuItems;
		unsigned int _fReturnSize;
		unsigned int _returnSize;
		unsigned int _searchXdefaults;
		unsigned int _searchGtkdefaults;
		unsigned int _searchAppdefaults;
		unsigned int _searchMenus;
		unsigned int _searchStyles;
		unsigned int _appendSearch;						// concatenate successive searches = 1
		unsigned int _defaultColourType;
		unsigned int _fallbackColourType;
		unsigned int _execOnModify;
		unsigned int _shadeReplace;
		unsigned int _generateHue;
		float _shadeOffset;
		unsigned int _saveResourceMods;
		unsigned int _undoFileNum;
		unsigned int _useFirstResource;					// 1 replacement subsititutes all similar resources
		unsigned int _keepOldSearchTargets;
		unsigned int _searchType;						// search resource type
		
		// utility functions
		int listThemes ( char *, void * );
		int listApps ( char * );
		int listAliases ( char *, void * );
		char * currentTheme ( char *, void * );
		int readPrefs ( char * );
		int writePrefs ( char * );
		char * tablet ( char *, char *, char * );		// stone tablet...
		int menu ( char *, char *, char *, char * );	// menu services...
		int reportColour ( ColourSpecConvert * );
		void reportColourError ( char *, int );			// colour, error
		void cleanOperate ( void );						// wipe list of items to operate on
		void cleanSearchInFiles ( void );				// wipe list of files to search
		void cleanSelApps ( void );						// wipe selected apps list
		void cleanSelThemes ( void );					// wipe selected themes list
		void cleanAvailable ( void );					// wipe apps and themes lists
		void cleanReplace ( void );
		void memError ( int, char * );
		char * shadeColour ( char *, char * );			// derivedColour from specified and replacement resources
		char * formatColour ( char * );					// replace user-defined substitute colour by one with type _defaultColourType
		int substituteResource ( void );				// modify resource files
		int execOnModify ( void );						// fire up app on mod if true
		unsigned int addColourTarget ( char *, ColourSpecConvert *, unsigned int,
										char *, char *, char *,
										float, float, float, float, float, float );	
								// see the implementation

		// member functions called by callback functions
		void quit ( Widget, XtPointer );
		int find ( Widget, XtPointer );					// specify things to search for
		int opts ( Widget, XtPointer );					// change rfe's behaviour
		int seln ( Widget, XtPointer );					// choose files to search in
		int view ( Widget, XtPointer );					// things to look at
		int exec ( Widget, XtPointer );					// actions rfe can take

		// callback functions
		static void quitCallback ( Widget, XtPointer, XtPointer );
		static void findCallback ( Widget, XtPointer, XtPointer );
		static void optsCallback ( Widget, XtPointer, XtPointer );
		static void selnCallback ( Widget, XtPointer, XtPointer );
		static void viewCallback ( Widget, XtPointer, XtPointer );
		static void execCallback ( Widget, XtPointer, XtPointer );
};

extern char **environ;

#endif
