/* 
   ColourSpecConvert.h: interconvert #, rgb. rgbi, named colours

   Copyright (C) Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __ColourSpecConvert_h
#define __ColourSpecConvert_h

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../Global.h"

enum COLOUR_SPEC { OK, HASH, RGB, RGBI, NAMED, INTEGER, ERR_STRINGTOOLONG,
			ERR_COLOURTYPE, ERR_DEC, ERR_HEX, ERR_LETTER, 
			ERR_SEPARATOR, ERR_DP, ERR_MISSING_DIGIT, ERR_RGBTXT,
			ERR_FILENOTFOUND, ERR_COLOURSEP, ERR_RESPATH,
			ERR_COLOURDEFSLIST, ERR_PROGRAM };

class ColourSpecConvert
{
	public:
		// constructor
		ColourSpecConvert();
		// destructor
		~ColourSpecConvert();

		int setColour ( char * );	// set the colour
		char * hashColour ( void );	// and get equivalents...
		char * rgbColour ( void );
		char * rgbiColour ( void );
		char * namedColour ( void );
		char * intColour ( void );
		char * analyseColour ( void );				
		char * aliasColour ( void );		
		unsigned int redInt ( void );	// 16-bit
		unsigned int greenInt ( void );
		unsigned int blueInt ( void );
		float redF ( void );
		float greenF ( void );
		float blueF ( void );
		void setReturnSize ( int );
		void setfReturnSize ( int );
		int setRgbTxtPath ( char * );	// /usr/X11/lib/rgb.txt
		int setResourceFilePath ( char * ); // ~/.blackhome/config

	private:
		struct Colour
		{
			unsigned int red;
			unsigned int green;
			unsigned int blue;
			unsigned int red8;
			unsigned int green8;
			unsigned int blue8;
			float redF, greenF, blueF;
			char namedColour [ MAXCHARS ]; // rgb.txt name
			char analysis [ MAXCHARS ]; // component description via rules
			char alias [ MAXCHARS ] ;  // .colourAliasList name	
		} colour;

    		char _colourString [ MAXCHARS ]; // named / colour string
		char _rgbDatabase [ MAXCHARS ];
		char _resourceFilePath [ MAXCHARS ];
		unsigned int _colourDataLength; // colour digits plus separators
		int _separatorCount;
		int _dpCount;
		int _returnSize;	// 4, 8, 16 bits per colour
		int _f_returnSize;	// rgbi floating point truncation
		
		int checkSpecType ( char * );
		int checkNamedColourByChar ( char * );
		int checkNamedColourByInt ( void  );
		int trimSpaces ( char * );					
		int checkDec ( char );
		int checkHex ( char );
		int checkChar ( char );
		int checkSeparator ( unsigned int );
		int checkDp ( unsigned int );
		int separateColour ( char *, int);
		int checkColourAlias ( char *, float, float );
		int describeColour ( char * );				
		unsigned int decHex ( unsigned int );
		unsigned int hexDec ( unsigned int );
		char * describeShade ( float );
		int maxIntMagnitude ( unsigned int, unsigned int, unsigned int );
};

#endif


// notes
// supplied colour string should be less than MAXCHARS bytes
// (no ridiculous names or aliases).
