/*
   colourSpecConvert.cc: colour spec conversion class implementation

   Copyright (C) Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ColourSpecConvert.h"
#include <math.h>

ColourSpecConvert::ColourSpecConvert(void)
{
	int count;
	struct Colour colour;

	colour.red = colour.green = colour.blue = 0;
	colour.red8 = colour.green8 = colour.blue8 = 0;

	for (count = 0; count < MAXCHARS; count++ )
		_colourString [ count ] = 0;

	_separatorCount = 0;
	_dpCount = 0;
	_returnSize = 12;	// default bits per colour
	_f_returnSize = 6;	// default digits after decimal point
	strcpy ( _rgbDatabase, "/usr/X11/lib/rgb.txt" );
	strcpy ( _resourceFilePath, "../../" );
}

ColourSpecConvert::~ColourSpecConvert(void)
{
}

// validate supplied string and set internal integer representation
int ColourSpecConvert::setColour ( char * colourString )
{	// _colourString is _reserved_ until this function returns
	unsigned long count;
	unsigned int startPtr;
	int colourType;
	char _redString [ MAXCHARS ];
	char _greenString [ MAXCHARS ];
	char _blueString [ MAXCHARS ];
	int error;

	colour.namedColour [ 0 ] = 0;
	colour.analysis [ 0 ] = 0;
	colour.alias [ 0 ] = 0;

	if ( strlen ( colourString ) >= MAXCHARS )
		return ERR_STRINGTOOLONG;
	else
	{
		trimSpaces ( colourString );

		if (strlen ( _colourString ) == 0 )
			return ERR_COLOURTYPE;
			
		colourType = checkSpecType ( _colourString );

		if ( colourType > INTEGER )
			return colourType;
	}

	// after this point the internal _colourString is set...
	if ( colourType == HASH )
	{
		startPtr = 1;
		_colourDataLength = strlen ( _colourString ) - startPtr;
		_separatorCount = 0;

		switch ( _colourDataLength )
		{
			case 1:
			case 2:
			case 4:
			case 5:
			case 7:
			case 8:
			case 10:
			case 11:
				return ERR_MISSING_DIGIT;
				break;
			case 3:
				_colourDataLength = 1;
				break;
			case 6:
				_colourDataLength = 2;
				break;
			case 9:
				_colourDataLength = 3;
				break;
			case 12:
				_colourDataLength = 4;
				break;
			case 14:
				return ERR_RGBTXT;
				break;
			case 15:
				return ERR_FILENOTFOUND;
				break;
			case 16:
				return ERR_COLOURSEP;
				break;
			case 17:
				return ERR_RESPATH;
				break;
			case 18:
				return ERR_COLOURDEFSLIST;
				break;
			default:
				_colourDataLength = 0;
				return ERR_PROGRAM;
		}

		for ( count = startPtr; count < strlen ( _colourString );  count++ )
		{
			if ( checkHex ( _colourString [ count ] ) != 0 )
			{
				if ( checkSeparator ( _colourString [ count ] ) == 0 )
					return ERR_SEPARATOR;
				else
					return ERR_COLOURTYPE;
			}
		} // valid hex colour spec
	}

	if ( colourType == RGB )
	{
		startPtr = 4;
		_separatorCount = 0;

		for ( count = startPtr; count < strlen ( _colourString );  count++ )
		{
			if ( checkHex ( _colourString [ count ] ) != 0 )
			{
				if ( checkSeparator ( _colourString [ count ] ) != 0 )
					return ERR_SEPARATOR;
				else
				{
					if ( _separatorCount > 2 )
						return ERR_SEPARATOR;
				}				
			}
		} // valid hex colour spec

		_colourDataLength = strlen ( _colourString ) - startPtr;

		count = startPtr;
		unsigned int count2 = 0;
	
		while ( checkSeparator ( _colourString [ count ] ) != 0 )
		{
			_redString [ count2 ] = _colourString [ count ];
			count++;
			count2++;
		}
			
		_redString [ count2 ] = 0;
		count++;
		count2 = 0;

		while ( checkSeparator ( _colourString [ count ] ) != 0 )
		{
			_greenString [ count2 ] = _colourString [ count ];
			count++;
			count2++;
		}
		
		_greenString [ count2 ] = 0;
		count++;
		count2 = 0;

		while ( count < strlen ( _colourString ) )
		{
			_blueString [ count2 ] = _colourString [ count ];
			count++;
			count2++;
		}

		_blueString [ count2 ] = 0;
		_colourDataLength = 0;

		if ( strlen ( _redString ) > _colourDataLength )
			_colourDataLength = strlen ( _redString );

		if ( strlen ( _greenString ) > _colourDataLength )
			_colourDataLength = strlen ( _greenString );

		if ( strlen ( _blueString ) > _colourDataLength )
			_colourDataLength = strlen ( _blueString );

		strcpy ( _colourString, "rgb:" );

		for ( count = 0; count < ( _colourDataLength - strlen ( _redString ) ); count++ )
			strcat ( _colourString, "0" );
		
		strcat ( _colourString, _redString );
		strcat ( _colourString, "/");
			
		for ( count = 0; count < ( _colourDataLength - strlen ( _greenString ) ); count++ )
			strcat ( _colourString, "0" );
					
		strcat ( _colourString, _greenString );
		strcat ( _colourString, "/" );

		for ( count = 0; count < ( _colourDataLength - strlen ( _blueString ) ); count++ )
			strcat ( _colourString, "0" );
		
		strcat ( _colourString, _blueString );
	}		

	if ( colourType == RGBI )
	{
		startPtr = 5;
		_separatorCount = 0;
		_dpCount = 0;
		_colourDataLength = strlen ( _colourString ) - startPtr;
		
                for ( count = startPtr; count < strlen ( _colourString ); count++ )
                {       
			if ( checkDec ( _colourString [ count ] ) == ERR_DEC )
			{
				if ( checkSeparator ( _colourString [ count ] ) == ERR_SEPARATOR )		
				{
					if ( checkDp ( _colourString [ count ] ) == ERR_DP )
						return ERR_DP;
				}
			}
		}

		if ( _separatorCount > 2 )
			return ERR_SEPARATOR;

		if ( _dpCount > 3 )
			return ERR_DP;
	}


	if ( colourType != NAMED ) // colours were already separated
	{	// checkSpecType does checkNamedColour by supplied name...
		error = 0;
		separateColour ( _colourString, colourType );
		error = checkNamedColourByInt(); // see if there is a name to match

		if ( error )
			return error;	
	}

	if ( colourType == INTEGER )
	{
		if ( strlen ( _colourString ) < 5 )	// 3 digits, 2 spaces
			return ERR_MISSING_DIGIT;						

		// force the data size to avoid unnecessary digit promotion
		if ( maxIntMagnitude ( colour.red, colour.green, colour.blue ) < ( pow ( 16, ( _returnSize / 4 ) ) - 1 ) )
			_colourDataLength = (unsigned int) ( _returnSize / 4 );
	}

	if ( ( colour.red > 65535 ) || ( colour.green > 65535 ) || ( colour.blue> 65535 ) )
		return ERR_COLOURTYPE;

	describeColour( _colourString );	
	return 0;	
}

int ColourSpecConvert::checkSpecType ( char * _colourString )
{
	int returnType;
	unsigned int count;

	if ( strlen ( _colourString ) < 3 )
	{
		for ( count = 0; count < strlen ( _colourString ); count++ )
		{
			if ( checkDec ( _colourString [ count ] ) != 0 )
				if ( _colourString [ count ] != ' ' )
					return ERR_MISSING_DIGIT;
		}
	}

	if ( strstr ( _colourString, "#" ) != NULL )
		return HASH;

	if ( strstr ( _colourString, "rgb:" ) != NULL )
		return RGB;
		
	if ( strstr ( _colourString, "rgbi:" ) != NULL )
		return RGBI;

	// now try to find it in the rgb database (?!)
	if ( checkNamedColourByChar ( _colourString ) == 0 )
		return NAMED;

	// if it isn't in rgb.txt, see if it is a decimal spec.
	// fail above just means 'name lookup failed'
	returnType = INTEGER;
	_separatorCount = 0;
	
	for ( count = 0; count < strlen ( _colourString ); count++ )
	{
		if ( checkDec ( _colourString [ count ] ) != 0 ) // presume we found a separator
			_separatorCount++;
	}

	for ( count = 0; count < strlen ( _colourString ); count++ )
	{
		if ( checkDec ( _colourString [ count ] ) != 0 )
		{
			if ( _colourString [ count ] != ' ' )
			{
				if ( checkSeparator ( _colourString [ count ] ) == ERR_SEPARATOR )
				{
					if ( checkChar ( _colourString [ count ] ) == ERR_LETTER )
						return ERR_LETTER;
					else
						returnType = OK;
				}
			}
		}
	}

	return returnType;
}

int ColourSpecConvert::checkDec ( char digit )
{
	if ( ( digit > 47 ) && ( digit < 58 ) )
		return 0;

	return ERR_DEC;
}


int ColourSpecConvert::checkChar ( char digit )
{
	if ( ( digit > 64 ) && ( digit < 91 ) )
		return 0;
		
	if ( ( digit > 96 ) && ( digit < 123 ) )
		return 0;
		
	return ERR_LETTER;
}

int ColourSpecConvert::checkHex ( char digit )
{
	if ( checkDec ( digit ) == 0 )
		return 0;
	
	if ( ( digit > 64 ) && ( digit < 71 ) )
		return 0;

	if ( ( digit > 96 ) && ( digit < 103 ) )
		return 0;

	return ERR_HEX;
}

int ColourSpecConvert::checkSeparator ( unsigned int digit )
{
	if ( (char) digit == '/' )
	{
		_separatorCount++;
		return 0;
	}
	else
		return 	ERR_SEPARATOR;
}

int ColourSpecConvert::checkDp ( unsigned int digit )
{
	if ( (char) digit == '.' )
	{
		_dpCount++;
		return 0;
	}
	else
		return ERR_DP;
}

int ColourSpecConvert::separateColour ( char * _colourString, int colourType )
{	// set colour.red, green and blue
	unsigned int count, count2;
	unsigned int endPtr;
	unsigned int red, green, blue;
	unsigned int error;
	float _f_red, _f_green, _f_blue;
	char _redString [ MAXCHARS ];
	char _greenString [ MAXCHARS ];
	char _blueString [ MAXCHARS ];
	int stripSeparator = 0;
	
	error = ERR_COLOURSEP; // unless declared false
	colour.red = colour.green = colour.blue = 0;
	
	if ( colourType == RGB )
		stripSeparator = 1;

	if ( ( colourType == HASH ) || ( colourType == RGB ) )
	{
		endPtr = strlen ( _colourString ) - 1;
		count2 = 0;

		for (count = 0; count < _colourDataLength; count++)
		{
			blue = decHex ( _colourString [ endPtr ] ); // decimal return
			colour.blue = colour.blue + ( blue * ( unsigned int ) pow (16, count ) );
			endPtr--;
		}

		if ( stripSeparator == 1 )
			endPtr--;
			
		for (count = 0; count < _colourDataLength; count++)
		{
			green = decHex ( _colourString [ endPtr ] ); // decimal return
			colour.green = colour.green + ( green * ( unsigned int ) pow (16, count ) );
			endPtr--;
		}
		
		if ( stripSeparator == 1 )
			endPtr--;
			
		for (count = 0; count < _colourDataLength; count++)
		{
			red = decHex ( _colourString [ endPtr ] ); // decimal return
			colour.red = colour.red + ( red * ( unsigned int ) pow (16, count ) );
			endPtr--;
		}

		switch ( _colourDataLength )
		{
			case 1:

			case 2:			
				colour.red8 = colour.red;
				colour.green8 = colour.green;
				colour.blue8 = colour.blue;
				break;
			case 3:
				_f_red = ( float ) colour.red;
				_f_red += 0.5;
				colour.red8 = ( unsigned int )  _f_red / 16;
				_f_green = ( float ) colour.green;
				_f_green += 0.5;
				colour.green8 = ( unsigned int ) _f_green / 16;
				_f_blue = ( float ) colour.blue;
				_f_blue += 0.5;
				colour.blue8 = ( unsigned int ) _f_blue / 16;
				break;				
			case 4:
				_f_red = ( float ) colour.red / 256;
				_f_red += 0.5;
				colour.red8 = ( unsigned int ) _f_red;
				_f_green = ( float ) colour.green / 256;
				_f_green += 0.5;
				colour.green8 = ( unsigned int ) _f_green;
				_f_blue = ( float ) colour.blue / 256;
				_f_blue += 0.5;
				colour.blue8 = ( unsigned int ) _f_blue;					
				break;

			default:
				return ERR_COLOURSEP;
		}

		error = 0;
	}

	if ( colourType == RGBI )
	{
		count2 = 0;
		
		for (count = 5; count < _colourDataLength; count++)
		{
			while ( checkSeparator ( _colourString [ count ] ) != 0 )
			{
				_redString [ count2 ] = _colourString [ count ];
				count++;
				count2++;
			}

			_redString [ count2 ] = 0;
			count++;
			count2 = 0;

			while ( checkSeparator ( _colourString [ count ] ) != 0 )
			{
				_greenString [ count2 ] = _colourString [ count ];
				count++;
				count2++;
			}
					
			_greenString [ count2 ] = 0;
			count++;
			count2 = 0;

			while ( count < strlen ( _colourString ) )
			{
				_blueString [ count2 ] = _colourString [ count ];
				count++;
				count2++;
			}
		}

		_blueString [ count2 ] = 0;
		
		colour.red8 = ( unsigned int ) ( ( atof ( _redString ) * 255 ) + 0.5 );
		colour.green8 = ( unsigned int ) ( ( atof ( _greenString ) * 255 ) + 0.5 );
		colour.blue8 = ( unsigned int ) ( ( atof ( _blueString ) * 255 ) + 0.5 );

		colour.red = ( unsigned int ) ( ( atof ( _redString ) * 65535 ) + 0.5 );
		colour.green = ( unsigned int ) ( ( atof ( _greenString ) * 65535 ) + 0.5 );
		colour.blue = ( unsigned int ) ( ( atof ( _blueString ) * 65535 ) + 0.5 );
		_colourDataLength = 4;	// for max accuracy, use 16-bit values

		if ( colour.red > 65535 )
			return ERR_STRINGTOOLONG;
			
		if ( colour.green > 65535 )
			return ERR_STRINGTOOLONG;
			
		if ( colour.blue > 65535 )
			return ERR_STRINGTOOLONG;
			
		error = 0;
	}
	
	if ( colourType == INTEGER )
	{
		sscanf( _colourString, "%u %u %u", &red, &green, &blue);

		_colourDataLength = 1;

		if ( ( red > 15 ) || ( green > 15 ) || ( blue > 15 ) )
			_colourDataLength = 2;

		if ( ( red > 255 ) || ( green > 255 ) || ( blue > 255 ) )
			_colourDataLength = 3;

		if ( ( red > 4095 ) || ( green > 4095 ) || ( blue > 4095 ) )
			_colourDataLength = 4;

		if ( ( red < 256 ) && ( green < 256 ) && ( blue < 256 ) )
		{
			colour.red8 = red;
			colour.green8 = green;
			colour.blue8 = blue;
		}
		else
		{
			_f_red = ( float ) ( red ) / ( float ) 256;
			_f_red += 0.5;
			colour.red8 = ( unsigned int ) _f_red;
			_f_green = ( float ) ( green ) / ( float ) 256;
			_f_green += 0.5;
			colour.green8 = ( unsigned int ) _f_green;
			_f_blue = ( float ) ( blue ) / ( float ) 256;
			_f_blue += 0.5;
			colour.blue8 = ( unsigned int ) _f_blue;
		}

		colour.red = red;
		colour.green = green;
		colour.blue = blue;
		error = 0;
	}

	return error;
}

unsigned int ColourSpecConvert::hexDec ( unsigned int dec )
{	// accept decimal digit, return hexadecimal
	unsigned int hex;

	if ( dec <= 9 )
	{
		hex = dec;
		return hex;
	}
		
	switch ( dec )
	{
		case 10:
			hex = 'a';
			break;
		case 11:
			hex = 'b';
			break;
		case 12:
			hex = 'c';
			break;
		case 13:
			hex = 'd';
			break;
		case 14:
			hex = 'e';
			break;
		default:
			hex = 'f';
	}

	return hex;
}

unsigned int ColourSpecConvert::decHex ( unsigned int hex )
{	// accept hexadecimal digit, return decimal
	unsigned int dec;

	if ( hex <= '9' )
	{
		dec = hex - 48;
		return dec;
	}

	switch ( hex )
	{
		case 'A':
		case 'a':
			dec = 10;
			break;
		case 'B':
		case 'b':
			dec = 11;
			break;
		case 'C':
		case 'c':
			dec = 12;
			break;
		case 'D':
		case 'd':
			dec = 13;
			break;
		case 'E':
		case 'e':
			dec = 14;
			break;
		default:
			dec = 15;
	}

	return dec;
}

unsigned int ColourSpecConvert::redInt ( void )
{
	return colour.red;
} 

unsigned int ColourSpecConvert::greenInt ( void )
{
	return colour.green;
} 

unsigned int ColourSpecConvert::blueInt ( void )
{
	return colour.blue;
}

float ColourSpecConvert::redF ( void )
{
	colour.redF = ( float ) colour.red / ( float ) maxIntMagnitude ( colour.red, colour.green, colour.blue );
	return colour.redF;
}

float ColourSpecConvert::greenF ( void )
{
	colour.greenF = ( float ) colour.green / ( float ) maxIntMagnitude ( colour.red, colour.green, colour.blue );
	return colour.greenF;
}

float ColourSpecConvert::blueF ( void )
{
	colour.blueF = ( float ) colour.blue / ( float ) maxIntMagnitude ( colour.red, colour.green, colour.blue );
	return colour.blueF;
}

char * ColourSpecConvert::hashColour ( void )
{
	unsigned int count, maxlen;
	float round;
	int shift;
	float factor;
	float red, green, blue;
	char _redString [ MAXCHARS ];
	char _greenString [ MAXCHARS ];
	char _blueString [ MAXCHARS ];
	
	shift =  _colourDataLength - ( _returnSize / 4 );
	shift *= -1;
	factor = ( float ) pow ( 16, shift );

	if ( factor < 1 )	// scale numbers down to find return size
	{
		if (shift <= 0 )
			round = 0.5;
		else
			round = 0.0;
	
		red = ( float ) colour.red * factor + round;
		green = ( float ) colour.green * factor + round;
		blue = ( float ) colour.blue * factor + round;

		factor = pow ( 16, ( _returnSize / 4 ) );

		if ( red > ( factor - 1 ) )
			red = factor - 1;
		if ( green > ( factor - 1 ) )
			green = factor - 1;
		if ( blue > ( factor - 1) )
			blue = factor - 1;

	}
	else
	{
		red = ( float ) colour.red;
		green = ( float ) colour.green;
		blue = ( float ) colour.blue;
	}

	factor = 1;
	
	if ( ( _returnSize / 4 ) > ( int ) _colourDataLength )
		factor = ( _returnSize / 4 ) - _colourDataLength;

	if ( factor > 1 )
		factor = pow ( 16, factor );

	sprintf ( _redString, "%x",  ( unsigned int ) ( red * factor ) );
	sprintf ( _greenString, "%x", ( unsigned int ) ( green * factor ) );
	sprintf ( _blueString, "%x", ( unsigned int ) ( blue * factor ) );
	
	strcat ( _redString, "\0" );
	strcat ( _greenString, "\0" );
	strcat ( _blueString, "\0" );

	strcpy ( _colourString, "#" );

	maxlen = ( _returnSize / 4 ) - strlen ( _redString );
	count = 0;
		
	while ( count < maxlen )
	{
		strcat ( _colourString, "0" );
		count++;
	}

	strcat ( _colourString, _redString );

	maxlen = ( _returnSize / 4 ) - strlen ( _greenString );
	count = 0;

	while ( count < maxlen )
	{
		strcat ( _colourString, "0" );
		count++;
	}

	strcat ( _colourString, _greenString );

	maxlen = ( _returnSize / 4 ) - strlen ( _blueString );
	count = 0;
		
	while ( count < maxlen )
	{
		strcat ( _colourString, "0" );
		count++;
	}
	
	strcat ( _colourString, _blueString );

	strcat ( _colourString, "\0" );
	return _colourString;	
}

char * ColourSpecConvert::rgbColour ( void )
{
	unsigned int count, maxlen;
	int shift;
	float factor;
	float round;
	float red, green, blue;
	char _redString [ MAXCHARS ];
	char _greenString [ MAXCHARS ];
	char _blueString [ MAXCHARS ];	
	
	shift =  _colourDataLength - ( _returnSize / 4 );
	shift *= -1;
	factor = ( float ) pow ( 16, shift );

	if ( factor < 1 )	// scale numbers if greater than returnSize
	{
		if (shift <= 0 )
			round = 0.5;
		else
			round = 0.0;

		red = ( float ) colour.red * factor + round;
		green = ( float ) colour.green * factor + round;
		blue = ( float ) colour.blue * factor + round;

		factor = pow ( 16, ( _returnSize / 4 ) );

		if ( red > ( factor - 1 ) )
			red = factor - 1;
		if ( green > ( factor - 1 ) )
			green = factor - 1;
		if ( blue > ( factor - 1 ) )
			blue = factor - 1;
	}
	else
	{
		red = (float) colour.red;
		green = (float) colour.green;
		blue = (float) colour.blue;
	}

	factor = 1;
	
	if ( ( _returnSize / 4 ) > ( int ) _colourDataLength )
		factor = ( _returnSize / 4 ) - _colourDataLength;

	if ( factor > 1 )
		factor = pow ( 16, factor );

	sprintf ( _redString, "%x", ( unsigned int ) ( red * factor ) );
	sprintf ( _greenString, "%x", ( unsigned int ) ( green * factor ) );
	sprintf ( _blueString, "%x", ( unsigned int ) ( blue * factor ) );

	strcpy ( _colourString, "rgb:" );

	maxlen = ( _returnSize / 4 ) - strlen ( _redString );
	count = 0;

	while ( count < maxlen )
	{
		strcat ( _colourString, "0" );
		count++;
	}

	strcat ( _colourString, _redString );
	strcat ( _colourString, "/" );

	maxlen = ( _returnSize / 4 ) - strlen ( _greenString );
	count = 0;

	while ( count < maxlen )
	{
		strcat ( _colourString, "0" );
		count++;
	}

	strcat ( _colourString, _greenString );
	strcat ( _colourString, "/" );

	maxlen = ( _returnSize / 4 ) - strlen ( _blueString );
	count = 0;

	while ( count < maxlen )
	{
		strcat ( _colourString, "0" );
		count++;
	}

	strcat ( _colourString, _blueString );	
	strcat ( _colourString, "\0" );

	return _colourString;	
}

char * ColourSpecConvert::rgbiColour ( void )
{
	char numString [ 10 ];
	char controlString [ 10 ];
	float red, green, blue;
	float factor;

// if namedColour, bloody problem! if the return size is bigger than 256, have to
//multiply the number by pow(16, _colourDataLength )
// but then a _call_ to named Colour should set the ints to max, surely?
	factor = 1;
	if ( _colourDataLength > ( unsigned int ) ( _returnSize / 4 ) )
		factor = ( pow ( 16, _colourDataLength ) - 1 );
	else
		factor = ( pow ( 16, ( _returnSize / 4 ) ) - 1 );

	red = ( float ) colour.red / factor;
	green = ( float ) colour.green / factor;
	blue = ( float ) colour.blue / factor;

	strcpy ( controlString, "%." );
	sprintf ( numString, "%d", _f_returnSize );
	strcat ( controlString, numString );
	strcat ( controlString, "f" );
	strcpy ( _colourString, "rgbi:" );
	sprintf ( numString, controlString, red );
	strcat ( _colourString, numString);
	strcat ( _colourString, "/" );
	sprintf ( numString, controlString, green );
	strcat ( _colourString, numString);
	strcat ( _colourString, "/" );
	sprintf ( numString, controlString, blue );

	strcat ( _colourString, numString );
	return _colourString ;
}

char * ColourSpecConvert::namedColour ( void )
{
	return colour.namedColour;
}

int ColourSpecConvert::checkNamedColourByInt ( void )
{	// check for name match by int8 specification, set namedColour
	FILE *rgbTextFile;
	char c;
	char textString [ MAXCHARS ];
	char colourName2 [ MAXCHARS ];	
	int count;
	int error;
	unsigned int red, green, blue;

	if ( strlen ( _rgbDatabase ) == 0 )
		return ERR_RESPATH;
		
	if ( ( rgbTextFile = fopen ( _rgbDatabase, "r" ) ) == NULL )
		return ERR_FILENOTFOUND;
	else
	{
		c = 0;
		count = 0;
		while ( !feof ( rgbTextFile ) )
		{		
			fread ( &c, 1, 1, rgbTextFile );
			textString [ count ] = c;

			if ( c == 0x0a )	// got current line
			{
				error = 0;
				textString [ count + 1 ] = 0;

				if ( textString [ 0 ] != '!' ) // skip comments
				{
					sscanf(textString, "%d %d %d %s %s",&red, &green, &blue, colour.namedColour, colourName2 );

					if ( strlen ( colourName2 ) > 0 )
					{
						strcat ( colour.namedColour, " " );
						strcat ( colour.namedColour, colourName2 );
					}

					error = 0;
				}
				else
					error = 1;

				if ( ( red == colour.red8 ) && ( green == colour.green8 ) && ( blue == colour.blue8 ) && ( error == 0) )
				{	// get the name - will be chars or space
					fclose ( rgbTextFile );
					return 0;
				}
				else
				{	// clear strings for next colour
					colour.namedColour [ 0 ] = 0;
					colourName2 [ 0 ] = 0;
				}
				
				count = -1; // set to 0 on next line read
			} // end scan current line for name match to ints in rgb.txt
			count++; // read next char in from file
		} // end scan rgb.txt for the given colour name
	} // no such animal, pack up and go home.

	fclose ( rgbTextFile );
	colour.namedColour [ 0 ] = 0;
	return 0;
}

char * ColourSpecConvert::intColour ( void )
{
	int shift ;
	unsigned int redInt, greenInt, blueInt;
	float factor;
	float round;
	float red, green, blue;

	shift =  _colourDataLength - ( _returnSize / 4 );
	shift *= -1;
	factor = ( float ) pow ( 16, shift );

	if ( factor < 1 )	
	{
		if (shift <= 0 )
			round = 0.5;
		else
			round = 0.0;

		red = ( float ) colour.red * factor;
		green = ( float ) colour.green * factor;
		blue = ( float ) colour.blue * factor;

		redInt =  ( unsigned int ) (red + round);
		greenInt = ( unsigned int ) (green + round);
		blueInt = ( unsigned int ) (blue + round);

		factor = pow ( 16, (_returnSize/4));

		if ( redInt > ( unsigned int ) ( factor - 1 ) )
			redInt = ( unsigned int ) ( factor - 1 );
		if ( greenInt > ( unsigned int ) ( factor - 1 ) )
			greenInt = ( unsigned int ) ( factor - 1 );
		if ( blueInt > ( unsigned int ) ( factor - 1 ) )
			blueInt = ( unsigned int ) ( factor - 1 );
	}
	else
	{
		redInt = colour.red;
		greenInt = colour.green;
		blueInt = colour.blue;
	}

	sprintf ( _colourString, "%u %u %u", redInt, greenInt, blueInt );
	
	return _colourString;
}

char * ColourSpecConvert::aliasColour ( void )
{
	return colour.alias;
}

char * ColourSpecConvert::analyseColour ( void )
{
	return colour.analysis;
}

int ColourSpecConvert::setRgbTxtPath ( char * rgbTxt)
{
	if ( strlen ( rgbTxt ) == 0 )
		return ERR_RGBTXT;
		
	strcpy ( _rgbDatabase, rgbTxt );
	
	return 0;
}

int ColourSpecConvert::setResourceFilePath ( char * resourceFilePath )
{
	if ( strlen ( resourceFilePath ) == 0 )
		return ERR_RESPATH;
		
	strcpy ( _resourceFilePath, resourceFilePath );
	
	return 0;
}

int ColourSpecConvert::checkNamedColourByChar ( char * namedColour )
{	// see if the supplied named colour is in the file; get components
	FILE *rgbTextFile;	
	char c;	
	char textString [ EDITBUFSIZE ];
	char colourName [ MAXCHARS ];
	char colourName2 [ MAXCHARS ];	
	char tempName [ MAXCHARS ];
	int count;	
	unsigned int posnInLine;	
	unsigned int bufPos;
	unsigned int red, green, blue;

	_colourDataLength = 4;	// rgb.txt only specifies 8-bit values

	if ( strlen ( _rgbDatabase ) == 0 )
		return ERR_RGBTXT;
		
	if ( ( rgbTextFile = fopen ( _rgbDatabase, "r" ) ) == NULL )
		return ERR_FILENOTFOUND;	
	else	
	{
		c = 0;		
		count = 0;
		
		while ( !feof ( rgbTextFile ) )		
		{			
			fread ( &c, 1, 1, rgbTextFile );
			textString [ count ] = c;			
			
			if ( c == 0x0a )	// got current line			
			{
				textString [ count + 1 ] = 0;				
				if ( textString [ 0 ] != '!' ) // skip comments				
				{
					colourName [ 0 ] = 0;
					colourName2 [ 0 ] = 0;	// name extension after whitespace

					sscanf(textString, "%d %d %d %s %s",&red, &green, &blue,
							colourName, colourName2 );

					colour.red8 = red;
					colour.green8 = green;
					colour.blue8 = blue;

					colour.red = red * 257;
					colour.green = green * 257;
					colour.blue = blue * 257;

					if ( strlen ( colourName2 ) > 0 )
					{
						strcat ( colourName, " " );
						strcat ( colourName, colourName2 );
					}

					if ( strcmp ( namedColour, colourName ) == 0 )
					{
						strncpy ( colour.namedColour, colourName, sizeof (colour.namedColour ) );
						fclose ( rgbTextFile );
						return 0;					
					}

					// hmm, failed, extract spaces from found name					
					posnInLine = 0;

					for ( bufPos = 0; bufPos < strlen ( colourName ); bufPos++ )					
					{
						if ( colourName [ bufPos] != ' ' )						
						{							
							tempName [posnInLine ] = colourName [ bufPos ];							
							posnInLine++;						
						}					
					}

					tempName [ posnInLine ] = 0 ;

					// any better luck this time?  					
					if ( strcmp ( namedColour, tempName ) == 0 )					
					{
						strncpy ( colour.namedColour, colourName, sizeof (colour.namedColour ) );
						fclose ( rgbTextFile );
						return 0;					
					}					
					
					// finally try lc'ing the lot - another i18n problem
					
					for ( bufPos = 0; bufPos < strlen ( namedColour ); bufPos++ )
					{
						if ( ( namedColour [ bufPos ] > 64 ) && ( namedColour [ bufPos ] < 91 ) )
							namedColour [ bufPos ] += 32;
					}

					for ( bufPos = 0; bufPos < strlen ( tempName ); bufPos++ )
					{
						if ( ( tempName [ bufPos ] > 64 ) && ( tempName [ bufPos ] < 91 ) )
							tempName [ bufPos ] += 32;
					}

					if ( strcmp ( namedColour, tempName ) == 0 )					
					{
						strncpy ( colour.namedColour, colourName, sizeof (colour.namedColour ) );
						fclose ( rgbTextFile );
						return 0;					
					}
										
					if ( strcmp ( namedColour, tempName ) == 0 )					
					{
						strncpy ( colour.namedColour, colourName, sizeof (colour.namedColour ) );
						fclose ( rgbTextFile );
						return 0;					
					}					

					if ( strlen( colourName ) == 0 ) // run out!					
					{
						fclose ( rgbTextFile );
						_colourDataLength = 0;
						return ERR_COLOURTYPE;					
					}
				} // end extract info from current line
	
				count = -1; // set to 0 on next line read			
			} // end got current line
			
			count++; // read next char in from file
		} // end scan rgb.txt for the given colour name
	} // no such animal, pack up and go home.	
	
	fclose ( rgbTextFile );
	_colourDataLength = 0;
	return ERR_COLOURTYPE;
}

int ColourSpecConvert::trimSpaces ( char * colourString )
{	// trim sp. from colourString, return with _colourString set
	char temp [ MAXCHARS ];
	unsigned int count, count2;
	
	count = 0;
	
	while ( colourString [ count ] == ' ' )
		count++; // skip any leading spaces
	
	count2 = 0;

	while ( count < strlen ( colourString ) )
	{
		temp [ count2 ] = colourString [ count ];
		count++;
		count2++;
	}
	
	temp [ count2 ] = 0;
	count = 0;
	count2 = 0;
	
	// preserve essentials and strip trailing spaces...
	_separatorCount = 0; // since unused for '/ 'here, use for ' '

	while ( count < strlen ( temp ) )
	{
		if ( checkChar ( temp [ count ] ) == 0 )
		{
			_colourString [ count2 ] = temp [ count ];
			count2++;
		}
		else
		{
			if ( checkHex ( temp [ count ] ) == 0 )
			{
				_colourString [ count2 ] = temp [ count ];
				count2++;
			}
			else
			{	// check for intermediate spaces
				if ( temp [ count ] == ' ' )
				{
					if ( _separatorCount < 2 )
					{ // only providing there is a following digit
						if ( checkHex ( temp [ count + 1 ] ) == 0 ) // covers decimal also
						{
							_colourString [ count2 ] = temp [count ];
							_separatorCount++;
							count2++;
						}
					}
					else
					{
						if ( checkChar ( temp [ count + 1 ] ) == 0 ) // covers decimal also
						{
							_colourString [ count2 ] = temp [count ];
							count2++;
						}
					
					}
				}	
			}
		}

		if ( temp [ count ] == '#')
		{
			_colourString [ count2 ] = temp [ count ];
			count2++;
		}
		
		if ( temp [ count ] == ':')
		{
			_colourString [ count2 ] = temp [ count ];
			count2++;
		}
		
		if ( temp [ count ] == '/' )
		{
			_colourString [ count2 ] = temp [ count ];
			count2++;
		}			

		if ( temp [ count ] == '.' )
		{
			_colourString [ count2 ] = temp [ count ];		
			count2++;				 		 
		}
								 			 
		count++;
	}

	_colourString [ count2 ] = 0;	
	return 0;		
}

void ColourSpecConvert::setReturnSize ( int  bitsPerColour )
{
	_returnSize = bitsPerColour;
}

void ColourSpecConvert::setfReturnSize ( int numDigits )
{
	_f_returnSize = numDigits;
}

int ColourSpecConvert::describeColour ( char * _colourString )
{	// build description in reverse
	float r, g, b;
	char analysis [ MAXCHARS ];
	char description [ MAXCHARS ];
	
	analysis [ 0 ] = 0;
	description [ 0 ] = 0;
	
	switch ( _colourDataLength )
	{
		case 1:
			r = (float) colour.red / 15;
			g = (float) colour.green / 15;		
			b = (float) colour.blue / 15;
			break;
		case 2:
			r = (float) colour.red / 255;
			g = (float) colour.green / 255;
			b = (float) colour.blue / 255;
			break;
		case 3:
			r = (float) colour.red / 4095;
			g = (float) colour.green / 4095;
			b = (float) colour.blue / 4095;
			break;
		
		default:
			r = (float) colour.red / 65535;
			g = (float) colour.green / 65535;
			b = (float) colour.blue / 65535;
	}

	// grey

	if ( ( r == b ) || ( r == g ) || ( g == b ) )
	{
		strcat ( analysis, describeShade ( ( r + g + b ) / 3 ) );
		strcat ( analysis, " gray" );
	}

	// red

	if ( ( r > g ) && ( r > b ) )
	{
		strcat ( analysis, describeShade ( r ) );
		strcat ( analysis, " red" );

		checkColourAlias ( analysis, g, b );
		strcat ( description, colour.alias );
	}

	if ( ( r >= g ) && ( r > b ) && ( g > b ) )
	{	// red-orange-yellow if individual components > 0.5, otherwise brown
		strcat ( analysis, describeShade ( ( ( r + g ) / 2 ) ) );

		if ( ( r > 0.5 ) && ( g > 0.5 ) )
			strcat ( analysis, " yellow" );
		else
			strcat ( analysis, " brown" );
		
		checkColourAlias ( analysis, g, b );
		strcat ( description, colour.alias );
	}

	if ( ( r > g ) && ( r >= b ) && ( b > g ) )
	{	// magenta / purple
		strcat ( analysis, describeShade ( ( ( r + b ) / 2 ) ) );

		if ( ( r > 0.5 ) && ( b > 0.5 ) )		
			strcat ( analysis, " magenta" );
		else
			strcat ( analysis, " umber" ); // black - brown - dark violet
			
		checkColourAlias ( analysis, g, b );
		strcat ( description, colour.alias );
	}

	// green

	if ( ( g > r ) && ( g > b ) )
	{
		strcat ( analysis, describeShade ( g ) );
		strcat ( analysis, " green" );

		checkColourAlias ( analysis, r, b );
		strcat ( description, colour.alias );		
	}
	
	// green is green regardless of component value - we have a shortage of names

	if ( ( g >= r ) && ( g > b ) && ( r > b ) )
	{
		strcat ( analysis, describeShade ( ( ( r + g + b ) / 3 ) ) );
		
		if ( ( g > 0.5 ) & ( r > 0.5 ) )		
			strcat ( analysis, " citrus" );	
		else
			strcat ( analysis, " forest" );	// terrestrial verdure
		
		checkColourAlias ( analysis, r, b );
		strcat ( description, colour.alias );
	}

	if ( ( g > r ) && ( g >= b ) && ( b > r ) )
	{
		strcat ( analysis, describeShade ( ( ( r + g + b ) / 3 ) ) );

		if ( ( g > 0.5 ) & ( b > 0.5 ) )		
			strcat ( analysis, " aqua" );
		else
			strcat ( analysis, " seaweed" ); // aquatic verdure
			
		checkColourAlias ( analysis, r, b );
		strcat ( description, colour.alias );
	}
	
	// blue
	
	if ( ( b > r ) && ( b > g ) )
	{
		strcat ( analysis, describeShade ( b ) );
		strcat ( analysis, " blue" );
		
		checkColourAlias ( analysis, r, g );
		strcat ( description, colour.alias );
	}

	if ( ( b >= r ) && ( b > g ) && ( r > g ) )
	{
		strcat ( analysis, describeShade ( ( ( r + g + b ) / 3 ) ) );

		if ( ( r > 0.5 ) && ( b > 0.5 ) )		
			strcat ( analysis, " violet" );
		else
			strcat ( analysis, " purple" );

		checkColourAlias ( analysis, r, g );
		strcat ( description, colour.alias );
	}

	if ( ( b > r ) && ( b >= g ) && ( g > r ) )
	{
		strcat ( analysis, describeShade ( ( ( g + b ) / 2 ) ) );

		if ( ( g > 0.5 ) & ( b > 0.5 ) )		
			strcat ( analysis, " cyan" );
		else
			strcat ( analysis, " ultramarine" );
			
		checkColourAlias ( analysis, g, b );
		strcat ( description, colour.alias );
	}
		
	strcat ( analysis, "\0" );
	strcat ( description, "\0");

	if ( strlen ( analysis ) == 0 )
		strcat ( analysis, "\nColourSpecConvert::describeColour: no colour rule defined,\n" );

	unsigned int count = 0;
	
	while ( count < strlen ( analysis ) )
	{
		colour.analysis [ count ] = analysis [ count + 1 ];
		count++;
	}

	// now perform sensible substitutions on generated names
	// aliases for these are in '.colourAliasList

	if (strlen ( description ) != 0 ) // has a description in it
	{
		strcpy ( colour.alias, describeShade ( ( ( r + g + b ) / 3 ) ) );
		strcat ( colour.alias, " " );
		strcat ( colour.alias, description );
		strcat ( colour.alias, "\0" );		
	}

	// strip preceding space inserted by describeShade in colour.alias
	unsigned int count2;
	
	count = count2 = 0;
	
	while ( colour.alias [ count ] == ' ' )
		count++;
		
	while ( count < strlen ( colour.alias ) )
	{
		analysis [ count2 ] = colour.alias [ count ];
		count++;
		count2++;
	}
		
	analysis [ count2 ] = 0;
	strcpy ( colour.alias, analysis );

	return 0;
}

char * ColourSpecConvert::describeShade ( float colour )
{	// intensity by number of supplied colours - so pure primaries
	// index as 'pale'

	float intensity;	
	
	intensity = log ( colour + 1e-8 ); // avoid log(0)

	if ( intensity > log ( 0.93 ) )
		return " bright";
		
	if ( intensity > log ( 0.84 ) )
		return " pale";

	if ( intensity > log ( 0.73 ) )
		return " light";
		
	if ( intensity > log ( 0.58 ) )
		return " mid";

	if ( intensity > log ( 0.4 ) )
		return " deep";
		
	if ( intensity > log ( 0.2 ) )
		return " dark";
	
	return " shadow";
}

int ColourSpecConvert::checkColourAlias ( char * _colour, float col1, float col2)
{	// see if the supplied named colour has an alias
	// there are two descriptions for each name
	// distinguished by balance of components col1 and col2
	FILE *colourAlias;
	char colourDefsList [ MAXCHARS ];
	char c;
	char textString [ EDITBUFSIZE ];
	char baseColour [ MAXCHARS ];
	char colourVariant1 [ MAXCHARS ];
	char colourVariant2 [ MAXCHARS ];
	int count;
	int lineCount;

	if ( strlen ( _resourceFilePath ) == 0 )
		return ERR_RESPATH;
		
	strcpy ( colourDefsList, _resourceFilePath );
	strcat ( colourDefsList, "/.colourAliasList" );

	if ( ( colourAlias = fopen ( colourDefsList, "r" ) ) == NULL )
		return ERR_COLOURDEFSLIST;

	c = 0;
	count = 0;
	lineCount = 0;

	while ( !feof ( colourAlias ) )
	{
		fread ( &c, 1, 1, colourAlias );		
		textString [ count ] = c;
		if ( c == 0x0a )	// got line
		{
			textString [ count ] = 0;

			if ( ( textString [ 0 ] != '!' )  && ( strlen ( textString ) != 0 ) )
			{	// skip comments. empties
				if ( lineCount == 0 )
				{
					strcpy ( baseColour, " " ); // match to autogenerated preceding sp.
					strcat ( baseColour, textString );
				}
				
				if ( lineCount == 1 )
					strcpy ( colourVariant1, textString );
				
				if ( lineCount == 2 )
					strcpy ( colourVariant2, textString );

				strcat ( baseColour, "\0" );
				strcat ( colourVariant1, "\0" );
				strcat ( colourVariant2, "\0" );

				if ( lineCount == 2 )
				{ // have name and alternatives
					lineCount = -1;
					
					if ( strcmp ( baseColour, _colour ) == 0 )
					{	// matched autogenerated name
						if ( col1 > ( col2 / 2 ) ) // arbitrate balance
							strcpy ( colour.alias, colourVariant1 );
						else
							strcpy ( colour.alias, colourVariant2 );

						fclose ( colourAlias );
						return 0;
					}
				}
				
				lineCount++;
			} // end extract info from current line

			count = -1; // set to 0 on next line read
		} // end got current line
			
		count++; // read next char in from file
	} // end scan .colourAliasList for the given colour name

	fclose ( colourAlias );
	colour.alias [ 0 ] = 0;
	return  0; // fail silently so generated name is used
}


int ColourSpecConvert::maxIntMagnitude ( unsigned int red, unsigned int blue, unsigned int green )
{
	unsigned int max;
	
	max = red;
	
	if ( green > max )
		max = green;
		
	if ( blue > max )
		max = blue;
	
	if ( max < 16 )
		return 15;

	if ( max < 256 ) 
		return 255;
		
	if ( max < 4096 )
		return 4095;
		
	return 65535;
}
