/* 
   colourSpecConvertTest.cc - standalone colour spec converter

   Copyright (C) Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   // supplied colour spec _must_ be in quotes, e.g.
   // colourSpecConvertTest "seashell"
   // colourSpecConvertTest "123 45 237"
*/

#include "colourSpecConvert.cc"

void reportError ( char *, int );
int testColour(char *, ColourSpecConvert *);

int main ( int argc, char *argv[] )
{
	char test [ 80 ];
	ColourSpecConvert *myConvert = new ColourSpecConvert;
	int error;

	if ( argc < 2 )
	{
		printf ( "usage: colourSpecConvertTest \"<colourSpec>\"\n");
		return 1;
	}

	sprintf(test, "%s", argv[1]);

	// override default /usr/X11/lib	
	error = myConvert->setRgbTxtPath ( "/var/X11R6/lib/rgb.txt" );
		
	if ( error != 0 )
		reportError ( test, error );

	// override the default ~/.blackhome/config with the program's dir.
//	error = myConvert->setResourceFilePath ( "." );

//	if ( error != 0 )
//		reportError ( test, error );

	myConvert->setfReturnSize ( 4 );	// use 4 significant f.p. digits
	myConvert-> setReturnSize ( 8 );
 	// use 8 bits each for r,g, b 

	error = testColour ( test, myConvert );
	
	if ( error != 0 )
		reportError ( test, error );

	return 0;	
}

void reportError ( char * colourString, int error )
{
	printf ( "Error %d; in '%s': ", error, colourString );

	switch ( error )
	{
		case ERR_STRINGTOOLONG:
			printf ( "String length exceeds %d.\n", MAXCHARS );
			break;
			
		case ERR_COLOURTYPE:
			printf ( "Colour type unrecognised.\n" );
			break;
			
		case ERR_DEC:
			printf ( "unidentified decimal number.\n" );
			break;

		case ERR_HEX:
			printf ( "unidentified hexadecimal number.\n" );
			break;

		case ERR_LETTER:
			printf("expecting something recognisably in the alphabet.\n");
						
		case ERR_SEPARATOR:
			printf ( "missing or invalid separator.\n" );
			break;
			
		case ERR_DP:
			printf ( "surplus decimal points, confused...\n" );
			break;
			
		case ERR_MISSING_DIGIT:
			printf ( "missing digit.\n" );
			break;
			
		case ERR_RGBTXT:
			printf ( "path to rgb.txt unspecified.\n" );
			break;
			
		case ERR_FILENOTFOUND:
			printf ( "path error; cannot locate data file.\n" );
			break;
			
		case ERR_COLOURSEP:
			printf("ColourSpecConvert::separateColour failed.\n");

		case ERR_RESPATH:
			printf("resource file path not specified.\n");

		case ERR_COLOURDEFSLIST:
			printf("can't open .colourAliasList file.\n");
			
		default:
			printf ( "error in programmer.\n" );
	}
}

int testColour(char * test, ColourSpecConvert * myConvert)
{
        int error;

        error = myConvert->setColour ( test );

        if ( error == 0 )
        {
                sprintf(test, "%s", myConvert->intColour());
                printf("integer:     %s\n", test);

                sprintf(test, "%s", myConvert->hashColour());
                printf("truncated:   %s\n", test);

                sprintf(test, "%s", myConvert->rgbColour());
                printf("rgb:         %s\n", test);

                sprintf(test, "%s", myConvert->rgbiColour());
                printf("rgbi:        %s\n", test);

                sprintf(test, "%s", myConvert->namedColour());
                printf("named:       %s\n", test);

                sprintf(test, "%s", myConvert->analyseColour());
                printf("analysis:    %s\n", test);

                sprintf(test, "%s", myConvert->aliasColour());
                printf("alias:       %s\n", test);
		
        }
        else
                return error;
				
	return 0;
}
	
