/* 
   Menu.cc: simple Menu widget component

   Copyright (C) Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <X11/Intrinsic.h>			/* Intrinsics Definitions */
#include <X11/StringDefs.h>		/* Standard Name-String definitions */
#include "Menu.h"
#include <stdarg.h>


extern "C"
{
   void cancelCallback ( Widget, XtPointer, XtPointer );
   void menuEntrySelectCallback ( Widget, XtPointer, XtPointer );
}


Menu::Menu ( const char *name, Widget parent,
				 const char *menuButtonLabel,
				 const char *entryListLabel,
				 ... )
{
   Arg args[4];  
   int n;

   _popup = parent;

	XtSetArg ( args[0], XtNlabel, menuButtonLabel );

	_menuButton = XtCreateManagedWidget ( name, menuButtonWidgetClass,
														_popup, args, 1 );

	XtSetArg ( args[0], XtNlabel, entryListLabel );

	_menu = XtCreatePopupShell ( "menu", simpleMenuWidgetClass,
											_menuButton, args, 1 );

	_entry = XtCreateManagedWidget ( "line", smeLineObjectClass, _menu, NULL, 0 );

	char *menuItemText;
	va_list menuList;
	int numberOfMenuItems = 0;

	va_start(menuList,entryListLabel);

	while(va_arg(menuList,char *))
		numberOfMenuItems++;

	va_end(menuList);

	numberOfPointersToMemBlocks = numberOfMenuItems;
	arrayOfPointersToMemBlocks = new menuWidgetSet *[numberOfPointersToMemBlocks];

	va_start ( menuList, entryListLabel );
	menuItemText = va_arg ( menuList, char * );

	for ( n = 1; menuItemText; n++ )
	{
		arrayOfPointersToMemBlocks[n-1] = new menuWidgetSet;
		arrayOfPointersToMemBlocks[n-1]->n = n;
		arrayOfPointersToMemBlocks[n-1]->menu = this;

		XtSetArg ( args[0], XtNlabel, menuItemText );

		_entry = XtCreateManagedWidget ( menuItemText, smeBSBObjectClass,
													_menu, args, 1 );

		XtAddCallback(_entry, XtNcallback, menuEntrySelectCallback, ( XtPointer ) arrayOfPointersToMemBlocks[n-1] );

		menuItemText = va_arg ( menuList, char * );
	}

	va_end ( menuList );
	_state = MENU_NOT_USEFUL;
}


Menu::Menu ( const char *name, Widget parent,
				 const char *menuButtonLabel,
				 const char *entryListLabel,
				 int numberOfMenuItems,
				 int charArraySize,
				 char *itemArray )
{
   Arg args[1];  

   _popup = parent;

	XtSetArg ( args[0], XtNlabel, menuButtonLabel );

	_menuButton = XtCreateManagedWidget ( name, menuButtonWidgetClass,
														_popup, args, 1 );

	XtSetArg ( args[0], XtNlabel, entryListLabel );

	_menu = XtCreatePopupShell ( "menu", simpleMenuWidgetClass,	_menuButton, args, 1 );

	_entry = XtCreateManagedWidget ( "line", smeLineObjectClass, _menu, NULL, 0 );

	numberOfPointersToMemBlocks = numberOfMenuItems;
	arrayOfPointersToMemBlocks = new menuWidgetSet * [	numberOfPointersToMemBlocks ];

	for ( int item = 0; item < numberOfMenuItems; item++ )
	{
		arrayOfPointersToMemBlocks[ item ] = new menuWidgetSet;
		arrayOfPointersToMemBlocks[ item ]->n = item + 1;
		arrayOfPointersToMemBlocks[ item ]->menu = this;

		XtSetArg ( args[0], XtNlabel, ( char * ) &itemArray[ item * charArraySize ] );

		_entry = XtCreateManagedWidget ( ( char * ) &itemArray[ item * charArraySize ],
													smeBSBObjectClass, _menu, args, 1 );

		XtAddCallback(_entry, XtNcallback, menuEntrySelectCallback,
							( XtPointer ) arrayOfPointersToMemBlocks[ item ] );
	}

	_state = MENU_NOT_USEFUL;
}


Menu::~Menu()
{
	if ( numberOfPointersToMemBlocks > 0 )
	{
		for ( int count = 0; count < numberOfPointersToMemBlocks; count++ )
			delete arrayOfPointersToMemBlocks[ count ];

		delete[] arrayOfPointersToMemBlocks;
		arrayOfPointersToMemBlocks = 0;
		numberOfPointersToMemBlocks = 0;
	}
}


void Menu::setStateToNOTUseful ( void )
{
	_state = MENU_NOT_USEFUL;
}


MENU_STATE Menu::getState ( void )
{
	return _state;
}


// Callback functions

void Menu::menuEntrySelectCallback( Widget w, XtPointer clientData, XtPointer )
{
   menuWidgetSet *widgetPak = ( menuWidgetSet * ) clientData;
	Menu *managedMenu = widgetPak->menu;
	managedMenu->menuEntrySelect ( managedMenu, widgetPak->n  );
}


// class member functions for callback functions

void Menu::menuEntrySelect ( Menu *menu, int itemSelected )
{
	Menu *managedMenu = menu;
	managedMenu->setSelectedItem ( itemSelected );
	_state = MENU_OK_PRESSED; // exit on selecting an item
}

