/* 
   menuTest.cc test program for the menu component class

   Copyright (C) Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "Menu.h"

#include <X11/Xaw/Form.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <iostream>


void quitCallback ( Widget, XtPointer, XtPointer );
void menuCallback ( Widget, XtPointer, XtPointer );

XtAppContext app;


int main ( int argc, char *argv[] )
{
	Arg args[3];
   Widget shell, form, button, popup;

   shell = XtOpenApplication ( &app, "Input", NULL, 0, &argc, argv, NULL, shellWidgetClass, NULL, 0 );

   form = XtCreateManagedWidget ( "Form" , formWidgetClass, shell, NULL, 0 );

   button = XtCreateManagedWidget ( "Quit ", commandWidgetClass, form, NULL, 0 );

   XtAddCallback ( button, XtNcallback, quitCallback, NULL );

   popup = XtCreatePopupShell ( "Popup", shellWidgetClass, form, NULL, 0 );
/*
	// fixed string version
   Menu myMenu ( "myMenu", popup,
										"Select kill mode",
										"Fatality type?",
										"Trash hard drive",
										"Screw networking",
										"Rewrite boot sector",
										 NULL );
*/
	// pointer to char. array version
	int numMenuItems = 3;
	int lengthOfCharArray = 10;
	char strings [ numMenuItems ] [ lengthOfCharArray ];

	strcpy ( strings[0], "Plum" );
	strcpy ( strings[1], "Peach" );
	strcpy ( strings[2], "Pear" );

	Menu myMenu ( "myMenu", popup,
						"Access fruit!",
						"Today's Penguins' Choice",
						numMenuItems,
						lengthOfCharArray,
						( char * ) &strings );

	int n = 0;
	XtSetArg ( args [ n ], XtNfromVert, button ); n++;

   button = XtCreateManagedWidget ( "Menu", commandWidgetClass, form, args, n );   

   XtAddCallback ( button, XtNcallback, menuCallback, ( XtPointer ) &myMenu);
                                        
   XtRealizeWidget ( shell );
   XtAppMainLoop ( app );
}   

void quitCallback ( Widget, XtPointer, XtPointer )
{
   printf ( "quitCallback\n" );    
   exit ( 0 );
}

void menuCallback ( Widget, XtPointer clientData, XtPointer )
{
	Menu *myMenu = ( Menu * ) clientData;
	XtPopup ( ( Widget ) myMenu->_popup, XtGrabNone ); // popup widget
	XEvent menuEvent;

	while (myMenu->getState() == MENU_NOT_USEFUL )
	{
		XtAppNextEvent ( app, &menuEvent );
		XtDispatchEvent ( &menuEvent );
	}
  
	XtPopdown ( ( Widget ) myMenu->_popup );

	if ( myMenu->getState() == MENU_CANCEL_PRESSED )
	{
		printf ( "cancelled action:\n" );
		printf ( "return state %d\n", myMenu->getState() );
		myMenu->setStateToNOTUseful();
		printf ( "returned item %d\n", myMenu->getSelectedItem() );
	}

	if ( myMenu->getState() == MENU_OK_PRESSED )
	{
		printf ( "ok return state %d\n", myMenu->getState() );
		myMenu->setStateToNOTUseful();
		printf ( "returned item %d\n", myMenu->getSelectedItem() );
		printf ( "Penguin picks " );

		switch (myMenu->getSelectedItem())
		{
			case 1:
				printf ( "Plum\n" );
				break;
			case 2:
				printf ( "Peach\n" );
				break;
			case 3:
				printf ( "Pear\n" );
				break;
			default:
				printf ( "Nothing Whatsoever\n" );
		}
	}
}
