/* 
   text.cc simple text box component class implementation

   Copyright (C) Will 18, 1999.

   This file is part of blackhome.

   blackhome is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   blackhome is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with blackhome; see the file COPYING.  If not, write to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <X11/Intrinsic.h>			/* Intrinsics Definitions */
#include <X11/StringDefs.h>		/* Standard Name-String definitions */

#include "Text.h"

extern "C"
{
   void okCallback ( Widget, XtPointer, XtPointer );
   void cancelCallback ( Widget, XtPointer, XtPointer );
}


Text::Text ( const char *name, Widget parent )
{
   Arg args [ 7 ];   
   _popup = parent;

	_form = XtCreateManagedWidget ( name, formWidgetClass, _popup, NULL, 0 );

	XtSetArg ( args [ 0 ], XtNlabel, "" ); // supply text from calling code
	XtSetArg ( args [ 1 ], XtNtop, XawChainTop );
	XtSetArg ( args [ 2 ], XtNbottom, XawChainTop );
	XtSetArg ( args [ 3 ], XtNwidth, BOXWIDTH - 12 );
         
	_label = XtCreateManagedWidget ( "label", labelWidgetClass, _form, args, 4 );

    XtSetArg ( args [ 0 ], XtNeditType, XawtextEdit );
    XtSetArg ( args [ 1 ], XtNwrap, XawtextWrapWord );
    XtSetArg ( args [ 2 ], XtNtop, XawChainTop );
    XtSetArg ( args [ 3 ], XtNresize, True );
    XtSetArg ( args [ 4 ], XtNwidth, BOXWIDTH - 12 );
    XtSetArg ( args [ 5 ], XtNheight, BOXHEIGHT - 60 );
    XtSetArg ( args [ 6 ], XtNfromVert, _label );

	_asciiText = XtCreateManagedWidget ( "asciiText", asciiTextWidgetClass,
					_form, args, 7 );

	XtSetArg ( args[0], XtNlabel, "Ok" );
	XtSetArg ( args[1], XtNfromVert, _asciiText );
   
	_button = XtCreateManagedWidget ( "ok", commandWidgetClass,
                              	    _form, args, 2 );

	XtAddCallback ( _button, XtNcallback, &Text::okCallback,
					( XtPointer ) this );

	XtSetArg ( args[0], XtNlabel, "Cancel" );
	XtSetArg ( args[1], XtNfromVert, _asciiText );
	XtSetArg ( args[2], XtNfromHoriz, _button );
   
	_button = XtCreateManagedWidget ( "cancel", commandWidgetClass,
										_form, args, 3 );

	XtAddCallback ( _button, XtNcallback, &Text::cancelCallback, 
   					( XtPointer ) this );
                                      
	_state = TEXT_NOT_USEFUL;
}


Text::~Text()
{
	XtDestroyWidget ( _popup );
}


void Text::setStateToNOTUseful ( void )
{
	_state = TEXT_NOT_USEFUL;
}


TEXT_STATE Text::getState ( void )
{
	return _state;
}


void Text::setText ( char * setText )
{
	XtVaSetValues ( _asciiText, XtNstring, setText, NULL );
}


char *Text::getText ( void )
{
	return _text;
}


void Text::setLabelText ( char * labelText )
{  // set the text to be displayed on the label widget
	XtVaSetValues ( _label, XtNlabel, labelText, NULL );
}


// Callback functions

void Text::okCallback ( Widget w, XtPointer clientData, XtPointer )
{
	Text *managedText = ( Text * ) clientData;
	managedText->ok ( w, clientData );
}


void Text::cancelCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	Text *managedText = ( Text * ) clientData;
	managedText->cancel ( w, clientData );
}    


// class member functions for callback functions

void Text::ok ( Widget, XtPointer clientData )
{  	
	Arg args[1];

	XtSetArg ( args[0], XtNstring, &_text ); // place to put widget data
	XtGetValues ( _asciiText, args, 1 ); // get appropriate data from widget

	if( strlen(_text) == 0 )
	{
		// print some error text in the text widget since nothing typed in 
		XtVaSetValues ( _asciiText, XtNstring, 
						"Null entry:\npress cancel to exit", NULL );

		setStateToNOTUseful();
		return;
   }

	_state = TEXT_OK_PRESSED;
}


void Text::cancel ( Widget, XtPointer clientData )
{
	_state = TEXT_CANCEL_PRESSED;
	// now clear out the text widget
	XtVaSetValues ( _asciiText, XtNstring, "", NULL );
}

